#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.iran import Iran
from tests.common import CommonCountryTests


class TestIran(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Iran)

    def test_special_holidays(self):
        self.assertHoliday(
            "2023-08-01",
            "2023-08-02",
            "2026-03-01",
            "2026-03-02",
            "2026-03-03",
            "2026-03-04",
            "2026-03-05",
            "2026-03-06",
            "2026-03-07",
        )

    def test_islamic_revolution_day(self):
        name = "پیروزی انقلاب اسلامی"
        self.assertHolidayName(
            name,
            "2020-02-11",
            "2021-02-10",
            "2022-02-11",
            "2023-02-11",
            "2024-02-11",
            "2025-02-10",
        )
        self.assertHolidayName(name, self.full_range)

    def test_iranian_oil_industry_nationalization_day(self):
        name = "روز ملی شدن صنعت نفت ایران"
        self.assertHolidayName(
            name,
            "2020-03-19",
            "2021-03-19",
            "2022-03-20",
            "2023-03-20",
            "2024-03-19",
            "2025-03-19",
        )
        self.assertHolidayName(name, self.full_range)

    def test_last_day_of_year(self):
        self.assertHolidayName(
            "آخرین روز سال",
            "2005-03-20",
            "2009-03-20",
            "2013-03-20",
            "2017-03-20",
            "2021-03-20",
            "2025-03-20",
        )

    def test_nowruz(self):
        name = "جشن نوروز"
        name_holiday = "عیدنوروز"
        self.assertHolidayName(
            name,
            "2020-03-20",
            "2021-03-21",
            "2022-03-21",
            "2023-03-21",
            "2024-03-20",
            "2025-03-21",
        )
        self.assertHolidayName(name, self.full_range)

        self.assertHolidayName(
            name_holiday,
            "2020-03-21",
            "2020-03-22",
            "2020-03-23",
            "2021-03-22",
            "2021-03-23",
            "2021-03-24",
            "2022-03-22",
            "2022-03-23",
            "2022-03-24",
            "2023-03-22",
            "2023-03-23",
            "2023-03-24",
            "2024-03-21",
            "2024-03-22",
            "2024-03-23",
            "2025-03-22",
            "2025-03-23",
            "2025-03-24",
        )
        self.assertHolidayNameCount(name_holiday, 3, self.full_range)

    def test_islamic_republic_day(self):
        name = "روز جمهوری اسلامی"
        self.assertHolidayName(
            name,
            "2020-03-31",
            "2021-04-01",
            "2022-04-01",
            "2023-04-01",
            "2024-03-31",
            "2025-04-01",
        )
        self.assertHolidayName(name, self.full_range)

    def test_natures_day(self):
        name = "روز طبیعت"
        self.assertHolidayName(
            name,
            "2020-04-01",
            "2021-04-02",
            "2022-04-02",
            "2023-04-02",
            "2024-04-01",
            "2025-04-02",
        )
        self.assertHolidayName(name, self.full_range)

    def test_death_of_imam_khomeini(self):
        name = "رحلت حضرت امام خمینی"
        self.assertHolidayName(
            name,
            "2020-06-03",
            "2021-06-04",
            "2022-06-04",
            "2023-06-04",
            "2024-06-03",
            "2025-06-04",
        )
        self.assertHolidayName(name, range(1989, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 1989))

    def test_15_khordad_uprising(self):
        name = "قیام 15 خرداد"
        self.assertHolidayName(
            name,
            "2020-06-04",
            "2021-06-05",
            "2022-06-05",
            "2023-06-05",
            "2024-06-04",
            "2025-06-05",
        )
        self.assertHolidayName(name, self.full_range)

    def test_tasua(self):
        name = "تاسوعای حسینی"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-08-29",
            "2021-08-18",
            "2022-08-07",
            "2023-07-27",
            "2024-07-15",
            "2025-07-05",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_ashura(self):
        name = "عاشورای حسینی"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-08-30",
            "2021-08-19",
            "2022-08-08",
            "2023-07-28",
            "2024-07-16",
            "2025-07-06",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_arbaeen(self):
        name = "اربعین حسینی"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-10-08",
            "2021-09-27",
            "2022-09-17",
            "2023-09-06",
            "2024-08-25",
            "2025-08-14",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_death_of_prophet_muhammad_and_martyrdom_of_hasan_ibn_ali(self):
        name = "رحلت رسول اکرم؛شهادت امام حسن مجتبی علیه السلام"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-10-16",
            "2021-10-05",
            "2022-09-25",
            "2023-09-14",
            "2024-09-02",
            "2025-08-22",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_matyrdom_of_ali_al_rida(self):
        name = "شهادت امام رضا علیه السلام"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-10-17",
            "2021-10-07",
            "2022-09-27",
            "2023-09-16",
            "2024-09-04",
            "2025-08-24",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_matyrdom_of_hasan_al_askari(self):
        name = "شهادت امام حسن عسکری علیه السلام"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-10-25",
            "2021-10-15",
            "2022-10-05",
            "2023-09-24",
            "2024-09-12",
            "2025-09-01",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_birthday_of_muhammad_and_imam_jafar_al_sadiq(self):
        name = "میلاد رسول اکرم و امام جعفر صادق علیه السلام"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-11-03",
            "2021-10-24",
            "2022-10-14",
            "2023-10-03",
            "2024-09-21",
            "2025-09-10",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_matyrdom_of_fatima(self):
        name = "شهادت حضرت فاطمه زهرا سلام الله علیها"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-01-29",
            "2021-01-17",
            "2022-01-06",
            "2022-12-27",
            "2023-12-17",
            "2024-12-05",
            "2025-11-24",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_birthday_of_imam_ali(self):
        name = "ولادت امام علی علیه السلام و روز پدر"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-03-08",
            "2021-02-25",
            "2022-02-15",
            "2023-02-04",
            "2024-01-25",
            "2025-01-14",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_isra_and_miraj(self):
        name = "مبعث رسول اکرم (ص)"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-03-22",
            "2021-03-11",
            "2022-03-01",
            "2023-02-18",
            "2024-02-08",
            "2025-01-28",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_birthday_of_mahdi(self):
        name = "ولادت حضرت قائم عجل الله تعالی فرجه و جشن نیمه شعبان"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-04-09",
            "2021-03-29",
            "2022-03-18",
            "2023-03-08",
            "2024-02-25",
            "2025-02-14",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_matyrdom_of_imam_ali(self):
        name = "شهادت حضرت علی علیه السلام"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-05-15",
            "2021-05-04",
            "2022-04-23",
            "2023-04-12",
            "2024-04-01",
            "2025-03-22",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_eid_al_fitr(self):
        name = "عید سعید فطر"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-05-24",
            "2021-05-13",
            "2022-05-03",
            "2023-04-22",
            "2024-04-10",
            "2025-03-31",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_eid_al_fitr_holiday(self):
        name = "تعطیل به مناسبت عید سعید فطر"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-05-25",
            "2021-05-14",
            "2022-05-04",
            "2023-04-23",
            "2024-04-11",
            "2025-04-01",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_matyrdom_of_imam_jafar_al_sadiq(self):
        name = "شهادت امام جعفر صادق علیه السلام"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-06-17",
            "2021-06-06",
            "2022-05-27",
            "2023-05-16",
            "2024-05-04",
            "2025-04-24",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_eid_al_adha(self):
        name = "عید سعید قربان"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-07-31",
            "2021-07-21",
            "2022-07-10",
            "2023-06-29",
            "2024-06-17",
            "2025-06-06",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_eid_al_ghadeer(self):
        name = "عید سعید غدیر خم"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-08-08",
            "2021-07-29",
            "2022-07-18",
            "2023-07-07",
            "2024-06-25",
            "2025-06-14",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_2021(self):
        self.assertHolidaysInYear(
            2021,
            ("2021-01-17", "شهادت حضرت فاطمه زهرا سلام الله علیها"),
            ("2021-02-10", "پیروزی انقلاب اسلامی"),
            ("2021-02-25", "ولادت امام علی علیه السلام و روز پدر"),
            ("2021-03-11", "مبعث رسول اکرم (ص)"),
            ("2021-03-19", "روز ملی شدن صنعت نفت ایران"),
            ("2021-03-20", "آخرین روز سال"),
            ("2021-03-21", "جشن نوروز"),
            ("2021-03-22", "عیدنوروز"),
            ("2021-03-23", "عیدنوروز"),
            ("2021-03-24", "عیدنوروز"),
            ("2021-03-29", "ولادت حضرت قائم عجل الله تعالی فرجه و جشن نیمه شعبان"),
            ("2021-04-01", "روز جمهوری اسلامی"),
            ("2021-04-02", "روز طبیعت"),
            ("2021-05-04", "شهادت حضرت علی علیه السلام"),
            ("2021-05-13", "عید سعید فطر"),
            ("2021-05-14", "تعطیل به مناسبت عید سعید فطر"),
            ("2021-06-04", "رحلت حضرت امام خمینی"),
            ("2021-06-05", "قیام 15 خرداد"),
            ("2021-06-06", "شهادت امام جعفر صادق علیه السلام"),
            ("2021-07-21", "عید سعید قربان"),
            ("2021-07-29", "عید سعید غدیر خم"),
            ("2021-08-18", "تاسوعای حسینی"),
            ("2021-08-19", "عاشورای حسینی"),
            ("2021-09-27", "اربعین حسینی"),
            ("2021-10-05", "رحلت رسول اکرم؛شهادت امام حسن مجتبی علیه السلام"),
            ("2021-10-07", "شهادت امام رضا علیه السلام"),
            ("2021-10-15", "شهادت امام حسن عسکری علیه السلام"),
            ("2021-10-24", "میلاد رسول اکرم و امام جعفر صادق علیه السلام"),
        )

    def test_2022(self):
        self.assertHolidaysInYear(
            2022,
            ("2022-01-06", "شهادت حضرت فاطمه زهرا سلام الله علیها"),
            ("2022-02-11", "پیروزی انقلاب اسلامی"),
            ("2022-02-15", "ولادت امام علی علیه السلام و روز پدر"),
            ("2022-03-01", "مبعث رسول اکرم (ص)"),
            ("2022-03-18", "ولادت حضرت قائم عجل الله تعالی فرجه و جشن نیمه شعبان"),
            ("2022-03-20", "روز ملی شدن صنعت نفت ایران"),
            ("2022-03-21", "جشن نوروز"),
            ("2022-03-22", "عیدنوروز"),
            ("2022-03-23", "عیدنوروز"),
            ("2022-03-24", "عیدنوروز"),
            ("2022-04-01", "روز جمهوری اسلامی"),
            ("2022-04-02", "روز طبیعت"),
            ("2022-04-23", "شهادت حضرت علی علیه السلام"),
            ("2022-05-03", "عید سعید فطر"),
            ("2022-05-04", "تعطیل به مناسبت عید سعید فطر"),
            ("2022-05-27", "شهادت امام جعفر صادق علیه السلام"),
            ("2022-06-04", "رحلت حضرت امام خمینی"),
            ("2022-06-05", "قیام 15 خرداد"),
            ("2022-07-10", "عید سعید قربان"),
            ("2022-07-18", "عید سعید غدیر خم"),
            ("2022-08-07", "تاسوعای حسینی"),
            ("2022-08-08", "عاشورای حسینی"),
            ("2022-09-17", "اربعین حسینی"),
            ("2022-09-25", "رحلت رسول اکرم؛شهادت امام حسن مجتبی علیه السلام"),
            ("2022-09-27", "شهادت امام رضا علیه السلام"),
            ("2022-10-05", "شهادت امام حسن عسکری علیه السلام"),
            ("2022-10-14", "میلاد رسول اکرم و امام جعفر صادق علیه السلام"),
            ("2022-12-27", "شهادت حضرت فاطمه زهرا سلام الله علیها"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2023-02-04", "ولادت امام علی علیه السلام و روز پدر"),
            ("2023-02-11", "پیروزی انقلاب اسلامی"),
            ("2023-02-18", "مبعث رسول اکرم (ص)"),
            ("2023-03-08", "ولادت حضرت قائم عجل الله تعالی فرجه و جشن نیمه شعبان"),
            ("2023-03-20", "روز ملی شدن صنعت نفت ایران"),
            ("2023-03-21", "جشن نوروز"),
            ("2023-03-22", "عیدنوروز"),
            ("2023-03-23", "عیدنوروز"),
            ("2023-03-24", "عیدنوروز"),
            ("2023-04-01", "روز جمهوری اسلامی"),
            ("2023-04-02", "روز طبیعت"),
            ("2023-04-12", "شهادت حضرت علی علیه السلام"),
            ("2023-04-22", "عید سعید فطر"),
            ("2023-04-23", "تعطیل به مناسبت عید سعید فطر"),
            ("2023-05-16", "شهادت امام جعفر صادق علیه السلام"),
            ("2023-06-04", "رحلت حضرت امام خمینی"),
            ("2023-06-05", "قیام 15 خرداد"),
            ("2023-06-29", "عید سعید قربان"),
            ("2023-07-07", "عید سعید غدیر خم"),
            ("2023-07-27", "تاسوعای حسینی"),
            ("2023-07-28", "عاشورای حسینی"),
            ("2023-08-01", "تعطیلی عمومی"),
            ("2023-08-02", "تعطیلی عمومی"),
            ("2023-09-06", "اربعین حسینی"),
            ("2023-09-14", "رحلت رسول اکرم؛شهادت امام حسن مجتبی علیه السلام"),
            ("2023-09-16", "شهادت امام رضا علیه السلام"),
            ("2023-09-24", "شهادت امام حسن عسکری علیه السلام"),
            ("2023-10-03", "میلاد رسول اکرم و امام جعفر صادق علیه السلام"),
            ("2023-12-17", "شهادت حضرت فاطمه زهرا سلام الله علیها"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2023-02-04", "Birthday of Imam Ali"),
            ("2023-02-11", "Islamic Revolution Day"),
            ("2023-02-18", "Isra' and Mi'raj"),
            ("2023-03-08", "Birthday of Mahdi"),
            ("2023-03-20", "Iranian Oil Industry Nationalization Day"),
            ("2023-03-21", "Nowruz"),
            ("2023-03-22", "Nowruz Holiday"),
            ("2023-03-23", "Nowruz Holiday"),
            ("2023-03-24", "Nowruz Holiday"),
            ("2023-04-01", "Islamic Republic Day"),
            ("2023-04-02", "Nature's Day"),
            ("2023-04-12", "Martyrdom of Imam Ali"),
            ("2023-04-22", "Eid al-Fitr"),
            ("2023-04-23", "Eid al-Fitr Holiday"),
            ("2023-05-16", "Martyrdom of Imam Ja'far al-Sadiq"),
            ("2023-06-04", "Death of Imam Khomeini"),
            ("2023-06-05", "15 Khordad Uprising"),
            ("2023-06-29", "Eid al-Adha"),
            ("2023-07-07", "Eid al-Ghadeer"),
            ("2023-07-27", "Tasua"),
            ("2023-07-28", "Ashura"),
            ("2023-08-01", "Public Holiday"),
            ("2023-08-02", "Public Holiday"),
            ("2023-09-06", "Arbaeen"),
            ("2023-09-14", "Death of Prophet Muhammad and Martyrdom of Hasan ibn Ali"),
            ("2023-09-16", "Martyrdom of Ali al-Rida"),
            ("2023-09-24", "Martyrdom of Hasan al-Askari"),
            ("2023-10-03", "Birthday of Muhammad and Imam Ja'far al-Sadiq"),
            ("2023-12-17", "Martyrdom of Fatima"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.united_states_virgin_islands import UnitedStatesVirginIslands
from tests.common import CommonCountryTests


class TestUnitedStatesVirginIslands(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(UnitedStatesVirginIslands)

    def test_three_kings_day(self):
        self.assertHolidayName("Three Kings Day", (f"{year}-01-06" for year in self.full_range))

    def test_presidents_day(self):
        name = "Presidents' Day"
        self.assertHolidayName(
            name,
            "2020-02-17",
            "2021-02-15",
            "2022-02-21",
            "2023-02-20",
            "2024-02-19",
            "2025-02-17",
        )
        self.assertHolidayName(name, range(1971, self.end_year))
        self.assertHolidayName(name, (f"{year}-02-22" for year in range(self.start_year, 1971)))

    def test_transfer_day(self):
        self.assertHolidayName("Transfer Day", (f"{year}-03-31" for year in self.full_range))

    def test_holy_thursday(self):
        name = "Holy Thursday"
        self.assertHolidayName(
            name,
            "2020-04-09",
            "2021-04-01",
            "2022-04-14",
            "2023-04-06",
            "2024-03-28",
            "2025-04-17",
        )
        self.assertHolidayName(name, self.full_range)

    def test_good_friday(self):
        name = "Good Friday"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_monday(self):
        name = "Easter Monday"
        self.assertHolidayName(
            name,
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, self.full_range)

    def test_emancipation_day(self):
        self.assertHolidayName("Emancipation Day", (f"{year}-07-03" for year in self.full_range))

    def test_columbus_day_and_puerto_rico_friendship_day(self):
        name = "Columbus Day and Puerto Rico Friendship Day"
        self.assertHolidayName(
            name,
            "2020-10-12",
            "2021-10-11",
            "2022-10-10",
            "2023-10-09",
            "2024-10-14",
            "2025-10-13",
        )
        self.assertHolidayName(name, range(1937, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 1937))

    def test_liberty_day(self):
        self.assertHolidayName("Liberty Day", (f"{year}-11-01" for year in self.full_range))

    def test_christmas_second_day(self):
        self.assertHolidayName(
            "Christmas Second Day", (f"{year}-12-26" for year in self.full_range)
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "New Year's Day"),
            ("2022-01-06", "Three Kings Day"),
            ("2022-01-17", "Martin Luther King Jr. Day"),
            ("2022-02-14", "Valentine's Day"),
            ("2022-02-21", "Presidents' Day"),
            ("2022-03-17", "Saint Patrick's Day"),
            ("2022-03-31", "Transfer Day"),
            ("2022-04-14", "Holy Thursday"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-17", "Easter Sunday"),
            ("2022-04-18", "Easter Monday"),
            ("2022-05-08", "Mother's Day"),
            ("2022-05-30", "Memorial Day"),
            ("2022-06-19", "Father's Day; Juneteenth National Independence Day"),
            ("2022-06-20", "Juneteenth National Independence Day (observed)"),
            ("2022-07-03", "Emancipation Day"),
            ("2022-07-04", "Independence Day"),
            ("2022-09-05", "Labor Day"),
            ("2022-10-10", "Columbus Day and Puerto Rico Friendship Day"),
            ("2022-10-31", "Halloween"),
            ("2022-11-01", "Liberty Day"),
            ("2022-11-11", "Veterans Day"),
            ("2022-11-24", "Thanksgiving Day"),
            ("2022-12-24", "Christmas Eve"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Christmas Day (observed); Christmas Second Day"),
            ("2022-12-31", "New Year's Eve"),
        )

    def test_l10n_th(self):
        self.assertLocalizedHolidays(
            "th",
            ("2022-01-01", "วันขึ้นปีใหม่"),
            ("2022-01-06", "วันสมโภชพระคริสต์แสดงองค์"),
            ("2022-01-17", "วันมาร์ติน ลูเทอร์ คิง จูเนียร์"),
            ("2022-02-14", "วันวาเลนไทน์"),
            ("2022-02-21", "วันประธานาธิบดี"),
            ("2022-03-17", "วันนักบุญแพทริก"),
            ("2022-03-31", "วันส่งมอบดินแดน"),
            ("2022-04-14", "วันพฤหัสศักดิ์สิทธิ์"),
            ("2022-04-15", "วันศุกร์ประเสริฐ"),
            ("2022-04-17", "วันอาทิตย์อีสเตอร์"),
            ("2022-04-18", "วันจันทร์อีสเตอร์"),
            ("2022-05-08", "วันแม่"),
            ("2022-05-30", "วันรำลึก"),
            ("2022-06-19", "วันประกาศอิสรภาพแห่งชาติจูนทีนท์; วันพ่อ"),
            ("2022-06-20", "ชดเชยวันประกาศอิสรภาพแห่งชาติจูนทีนท์"),
            ("2022-07-03", "วันเลิกทาส"),
            ("2022-07-04", "วันประกาศอิสรภาพ"),
            ("2022-09-05", "วันแรงงาน"),
            ("2022-10-10", "วันโคลัมบัส และวันมิตรภาพกับเปอร์โตริโก"),
            ("2022-10-31", "วันฮาโลวีน"),
            ("2022-11-01", "วันเลิกทาส"),
            ("2022-11-11", "วันทหารผ่านศึก"),
            ("2022-11-24", "วันขอบคุณพระเจ้า"),
            ("2022-12-24", "วันคริสต์มาสอีฟ"),
            ("2022-12-25", "วันคริสต์มาส"),
            ("2022-12-26", "ชดเชยวันคริสต์มาส; วันคริสต์มาสวันที่สอง"),
            ("2022-12-31", "วันสิ้นปี"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.kosovo import Kosovo
from tests.common import CommonCountryTests


class TestKosovo(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Kosovo)

    def test_new_years_day(self):
        name = "Viti i Ri"
        self.assertHolidayName(
            name,
            (f"{year}-01-01" for year in self.full_range),
            (f"{year}-01-02" for year in self.full_range),
        )
        obs_dts = (
            "2010-01-04",
            "2011-01-03",
            "2012-01-03",
            "2016-01-04",
            "2017-01-03",
            "2022-01-03",
            "2022-01-04",
            "2023-01-03",
        )
        self.assertHolidayName(f"{name} (ditë pushimi e shtyrë)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_orthodox_christmas_day(self):
        name = "Krishtlindjet Ortodokse"
        self.assertHolidayName(name, (f"{year}-01-07" for year in self.full_range))
        obs_dts = (
            "2012-01-09",
            "2017-01-09",
            "2018-01-08",
            "2023-01-09",
            "2024-01-08",
        )
        self.assertHolidayName(f"{name} (ditë pushimi e shtyrë)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_independence_day(self):
        name = "Dita e Pavarësisë së Republikës së Kosovës"
        self.assertHolidayName(name, (f"{year}-02-17" for year in self.full_range))
        obs_dts = (
            "2013-02-18",
            "2018-02-19",
            "2019-02-18",
            "2024-02-19",
        )
        self.assertHolidayName(f"{name} (ditë pushimi e shtyrë)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_catholic_easter(self):
        name = "Pashkët Katolike"
        self.assertHolidayName(
            name,
            "2020-04-12",
            "2021-04-04",
            "2022-04-17",
            "2023-04-09",
            "2024-03-31",
            "2025-04-20",
        )
        self.assertHolidayName(name, self.full_range)
        # Easter Monday is always observed (Easter always falls on Sunday).
        self.assertHolidayName(
            f"{name} (ditë pushimi e shtyrë)",
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )

    def test_orthodox_easter(self):
        name = "Pashkët Ortodokse"
        self.assertHolidayName(
            name,
            "2020-04-19",
            "2021-05-02",
            "2022-04-24",
            "2023-04-16",
            "2024-05-05",
            "2025-04-20",
        )
        self.assertHolidayName(name, self.full_range)
        # Easter Monday is always observed (Easter always falls on Sunday).
        self.assertHolidayName(
            f"{name} (ditë pushimi e shtyrë)",
            "2020-04-20",
            "2021-05-04",
            "2022-04-25",
            "2023-04-17",
            "2024-05-06",
            "2025-04-21",
        )

    def test_constitution_day(self):
        name = "Dita e Kushtetutës së Republikës së Kosovës"
        self.assertHolidayName(name, (f"{year}-04-09" for year in self.full_range))
        obs_dts = (
            "2011-04-11",
            "2016-04-11",
            "2017-04-10",
            "2022-04-11",
            "2023-04-10",
        )
        self.assertHolidayName(f"{name} (ditë pushimi e shtyrë)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_international_workers_day(self):
        name = "Dita Ndërkombëtare e Punës"
        self.assertHolidayName(name, (f"{year}-05-01" for year in self.full_range))
        obs_dts = (
            "2010-05-03",
            "2011-05-02",
            "2016-05-02",
            "2021-05-03",
            "2022-05-03",
        )
        self.assertHolidayName(f"{name} (ditë pushimi e shtyrë)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_europe_day(self):
        name = "Dita e Evropës"
        self.assertHolidayName(name, (f"{year}-05-09" for year in self.full_range))
        obs_dts = (
            "2015-05-11",
            "2020-05-11",
            "2021-05-10",
            "2026-05-11",
        )
        self.assertHolidayName(f"{name} (ditë pushimi e shtyrë)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_catholic_christmas_day(self):
        name = "Krishtlindjet Katolike"
        self.assertHolidayName(name, (f"{year}-12-25" for year in self.full_range))
        obs_dts = (
            "2010-12-27",
            "2011-12-26",
            "2016-12-26",
            "2021-12-27",
            "2022-12-26",
        )
        self.assertHolidayName(f"{name} (ditë pushimi e shtyrë)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_eid_al_fitr(self):
        name = "Bajrami i Madh, dita e parë"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-05-24",
            "2021-05-13",
            "2022-05-02",
            "2023-04-21",
            "2024-04-10",
            "2025-03-30",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_eid_al_adha(self):
        name = "Bajrami i Vogël, dita e parë"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-07-31",
            "2021-07-20",
            "2022-07-09",
            "2023-06-28",
            "2024-06-16",
            "2025-06-06",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_2025(self):
        self.assertHolidaysInYear(
            2025,
            ("2025-01-01", "Viti i Ri"),
            ("2025-01-02", "Viti i Ri"),
            ("2025-01-07", "Krishtlindjet Ortodokse"),
            ("2025-02-17", "Dita e Pavarësisë së Republikës së Kosovës"),
            ("2025-03-30", "Bajrami i Madh, dita e parë (e vlerësuar)"),
            (
                "2025-03-31",
                "Bajrami i Madh, dita e parë (ditë pushimi e shtyrë, e vlerësuar)",
            ),
            ("2025-04-09", "Dita e Kushtetutës së Republikës së Kosovës"),
            ("2025-04-20", "Pashkët Katolike; Pashkët Ortodokse"),
            (
                "2025-04-21",
                "Pashkët Katolike (ditë pushimi e shtyrë); "
                "Pashkët Ortodokse (ditë pushimi e shtyrë)",
            ),
            ("2025-05-01", "Dita Ndërkombëtare e Punës"),
            ("2025-05-09", "Dita e Evropës"),
            ("2025-06-06", "Bajrami i Vogël, dita e parë (e vlerësuar)"),
            ("2025-12-25", "Krishtlindjet Katolike"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2024-01-01", "Viti i Ri"),
            ("2024-01-02", "Viti i Ri"),
            ("2024-01-07", "Krishtlindjet Ortodokse"),
            ("2024-01-08", "Krishtlindjet Ortodokse (ditë pushimi e shtyrë)"),
            ("2024-02-17", "Dita e Pavarësisë së Republikës së Kosovës"),
            (
                "2024-02-19",
                "Dita e Pavarësisë së Republikës së Kosovës (ditë pushimi e shtyrë)",
            ),
            ("2024-03-31", "Pashkët Katolike"),
            ("2024-04-01", "Pashkët Katolike (ditë pushimi e shtyrë)"),
            ("2024-04-09", "Dita e Kushtetutës së Republikës së Kosovës"),
            ("2024-04-10", "Bajrami i Madh, dita e parë (e vlerësuar)"),
            ("2024-05-01", "Dita Ndërkombëtare e Punës"),
            ("2024-05-05", "Pashkët Ortodokse"),
            ("2024-05-06", "Pashkët Ortodokse (ditë pushimi e shtyrë)"),
            ("2024-05-09", "Dita e Evropës"),
            ("2024-06-16", "Bajrami i Vogël, dita e parë (e vlerësuar)"),
            (
                "2024-06-17",
                "Bajrami i Vogël, dita e parë (ditë pushimi e shtyrë, e vlerësuar)",
            ),
            ("2024-12-25", "Krishtlindjet Katolike"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2024-01-01", "New Year's Day"),
            ("2024-01-02", "New Year's Day"),
            ("2024-01-07", "Orthodox Christmas Day"),
            ("2024-01-08", "Orthodox Christmas Day (observed)"),
            ("2024-02-17", "Independence Day"),
            ("2024-02-19", "Independence Day (observed)"),
            ("2024-03-31", "Catholic Easter"),
            ("2024-04-01", "Catholic Easter (observed)"),
            ("2024-04-09", "Constitution Day"),
            ("2024-04-10", "Eid al-Fitr (estimated)"),
            ("2024-05-01", "International Workers' Day"),
            ("2024-05-05", "Orthodox Easter"),
            ("2024-05-06", "Orthodox Easter (observed)"),
            ("2024-05-09", "Europe Day"),
            ("2024-06-16", "Eid al-Adha (estimated)"),
            ("2024-06-17", "Eid al-Adha (observed, estimated)"),
            ("2024-12-25", "Catholic Christmas Day"),
        )

    def test_l10n_sr(self):
        self.assertLocalizedHolidays(
            "sr",
            ("2024-01-01", "Nova godina"),
            ("2024-01-02", "Nova godina"),
            ("2024-01-07", "Pravoslavni Božić"),
            ("2024-01-08", "Pravoslavni Božić (slobodan dan)"),
            ("2024-02-17", "Dan Nezavisnosti Republike Kosova"),
            ("2024-02-19", "Dan Nezavisnosti Republike Kosova (slobodan dan)"),
            ("2024-03-31", "Katolički Uskrs"),
            ("2024-04-01", "Katolički Uskrs (slobodan dan)"),
            ("2024-04-09", "Dan Ustava Republike Kosova"),
            ("2024-04-10", "Fiter Bajram, prvi dan (procenjeno)"),
            ("2024-05-01", "Međunarodni Dan Rada"),
            ("2024-05-05", "Pravoslavni Uskrs"),
            ("2024-05-06", "Pravoslavni Uskrs (slobodan dan)"),
            ("2024-05-09", "Dan Evrope"),
            ("2024-06-16", "Kurban Bajram, prvi dan (procenjeno)"),
            ("2024-06-17", "Kurban Bajram, prvi dan (slobodan dan, procenjeno)"),
            ("2024-12-25", "Katolički Božić"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.uruguay import Uruguay
from tests.common import CommonCountryTests


class TestUruguay(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Uruguay)

    def test_special_holidays(self):
        self.assertHoliday(
            "1985-03-01",
            "1990-03-01",
            "1995-03-01",
            "2000-03-01",
            "2005-03-01",
            "2010-03-01",
            "2015-03-01",
            "2020-03-01",
        )

    def test_new_years_day(self):
        self.assertHolidayName("Año Nuevo", (f"{year}-01-01" for year in self.full_range))

    def test_cry_of_asencio(self):
        name = "Grito de Asencio"
        self.assertHolidayName(name, (f"{year}-02-28" for year in range(self.start_year, 1934)))
        self.assertNoHolidayName(name, range(1934, self.end_year))

    def test_labor_day(self):
        self.assertHolidayName(
            "Día de los Trabajadores",
            (
                f"{year}-05-01"
                for year in (*range(self.start_year, 1980), *range(1982, self.end_year))
            ),
            "1980-05-05",
            "1981-05-04",
        )

    def test_spain_day(self):
        name = "Día de España"
        self.assertHolidayName(name, (f"{year}-05-02" for year in range(self.start_year, 1933)))
        self.assertNoHolidayName(name, range(1933, self.end_year))

    def test_america_day(self):
        name = "Día de América"
        self.assertHolidayName(name, (f"{year}-05-25" for year in range(self.start_year, 1933)))
        self.assertNoHolidayName(name, range(1933, self.end_year))

    def test_democracy_day(self):
        name = "Día de la Democracia"
        self.assertHolidayName(name, (f"{year}-07-04" for year in range(self.start_year, 1933)))
        self.assertNoHolidayName(name, range(1933, self.end_year))

    def test_humanity_day(self):
        name = "Día de la Humanidad"
        self.assertHolidayName(name, (f"{year}-07-14" for year in range(self.start_year, 1933)))
        self.assertNoHolidayName(name, range(1933, self.end_year))

    def test_constitution_day(self):
        self.assertHolidayName(
            "Jura de la Constitución", (f"{year}-07-18" for year in self.full_range)
        )

    def test_independence_day(self):
        self.assertHolidayName(
            "Declaratoria de la Independencia", (f"{year}-08-25" for year in self.full_range)
        )

    def test_italy_day(self):
        name = "Día de Italia"
        self.assertHolidayName(name, (f"{year}-09-20" for year in range(self.start_year, 1933)))
        self.assertNoHolidayName(name, range(1933, self.end_year))

    def test_open_town_hall(self):
        name = "Cabildo Abierto"
        self.assertHolidayName(name, (f"{year}-09-21" for year in range(self.start_year, 1933)))
        self.assertNoHolidayName(name, range(1933, self.end_year))

    def test_beaches_day(self):
        name = "Día de las Playas"
        self.assertHolidayName(
            name, (f"{year}-12-08" for year in (*range(self.start_year, 1933), *range(1936, 1980)))
        )
        self.assertNoHolidayName(name, range(1933, 1936), range(1980, self.end_year))

    def test_christmas(self):
        self.assertHolidayName("Día de la Familia", (f"{year}-12-25" for year in self.full_range))

    def test_childrens_day(self):
        name = "Día de los Niños"
        self.assertNoHolidayName(name)
        self.assertBankHolidayName(name, (f"{year}-01-06" for year in self.full_range))

    def test_carnival(self):
        name = "Carnaval"
        self.assertNoHolidayName(name)
        self.assertBankHolidayName(
            name,
            "2018-02-12",
            "2018-02-13",
            "2019-03-04",
            "2019-03-05",
            "2020-02-24",
            "2020-02-25",
            "2021-02-15",
            "2021-02-16",
            "2022-02-28",
            "2022-03-01",
            "2023-02-20",
            "2023-02-21",
        )
        self.assertBankHolidayNameCount(name, 2, self.full_range)

    def test_landing_of_33_patriots(self):
        name = "Desembarco de los 33 Orientales"
        self.assertNoHolidayName(name)
        self.assertBankHolidayName(
            name,
            (
                f"{year}-04-19"
                for year in (*range(self.start_year, 1933), *range(1949, 1983), *range(1984, 1997))
            ),
        )
        self.assertBankHolidayName(
            name,
            "1983-04-18",
            "2000-04-17",
            "2018-04-23",
            "2019-04-22",
            "2020-04-19",
            "2021-04-19",
            "2022-04-18",
            "2023-04-17",
        )
        self.assertBankHolidayName(name, range(1949, self.end_year))
        self.assertNoBankHolidayName(name, range(1934, 1949))

    def test_tourism_week(self):
        name = "Semana de Turismo"
        self.assertNoHolidayName(name)
        self.assertBankHolidayName(
            name,
            "2020-04-06",
            "2020-04-07",
            "2020-04-08",
            "2020-04-09",
            "2020-04-10",
            "2021-03-29",
            "2021-03-30",
            "2021-03-31",
            "2021-04-01",
            "2021-04-02",
            "2022-04-11",
            "2022-04-12",
            "2022-04-13",
            "2022-04-14",
            "2022-04-15",
        )
        self.assertBankHolidayNameCount(name, 5, self.full_range)

    def test_battle_of_las_piedras(self):
        name = "Batalla de Las Piedras"
        self.assertNoHolidayName(name)
        self.assertBankHolidayName(
            name,
            (
                f"{year}-05-18"
                for year in (*range(self.start_year, 1933), *range(1942, 1982), *range(1984, 1997))
            ),
        )
        self.assertBankHolidayName(
            name,
            "1982-05-17",
            "1983-05-16",
            "1999-05-17",
            "2018-05-21",
            "2019-05-18",
            "2020-05-18",
            "2021-05-17",
            "2022-05-16",
            "2023-05-22",
        )
        self.assertBankHolidayName(name, range(1942, self.end_year))
        self.assertNoBankHolidayName(name, range(1933, 1942))

    def test_birthday_of_artigas(self):
        name = "Natalicio de Artigas"
        self.assertNoHolidayName(name)
        self.assertBankHolidayName(
            name,
            (
                f"{year}-06-19"
                for year in (
                    *range(self.start_year, 1933),
                    *range(1940, 1980),
                    *range(1982, 1997),
                    *range(1999, 2001),
                    *range(2002, self.end_year),
                )
            ),
        )
        self.assertBankHolidayName(
            name,
            "1980-06-23",
            "1981-06-22",
            "1997-06-23",
            "1998-06-22",
            "2001-06-18",
        )
        self.assertBankHolidayName(name, range(1940, self.end_year))
        self.assertNoBankHolidayName(name, range(1933, 1940))

    def test_cultural_diversity_day(self):
        name_1920 = "Día de la Raza"
        name_2014 = "Día de la Diversidad Cultural"

        self.assertNoHolidayName(name_1920)
        self.assertBankHolidayName(
            name_1920,
            (
                f"{year}-10-12"
                for year in (*range(self.start_year, 1933), *range(1937, 1982), *range(1984, 1997))
            ),
        )
        self.assertBankHolidayName(
            name_1920,
            "1982-10-11",
            "1983-10-10",
            "1999-10-11",
            "2013-10-12",
        )
        self.assertBankHolidayName(name_1920, range(1937, 2014))
        self.assertNoBankHolidayName(name_1920, range(1933, 1937), range(2014, self.end_year))

        self.assertNoHolidayName(name_2014)
        self.assertBankHolidayName(
            name_2014,
            "2014-10-12",
            "2018-10-15",
            "2019-10-12",
            "2020-10-12",
            "2021-10-11",
            "2022-10-10",
            "2023-10-16",
        )
        self.assertBankHolidayName(name_2014, range(2014, self.end_year))
        self.assertNoBankHolidayName(name_2014, range(self.start_year, 2014))

    def test_all_souls_day(self):
        name = "Día de los Difuntos"
        self.assertNoHolidayName(name)
        self.assertBankHolidayName(
            name,
            (
                f"{year}-11-02"
                for year in (
                    *range(self.start_year, 1933),
                    *range(1938, 1982),
                    *range(1984, 1999),
                    *range(2002, self.end_year),
                )
            ),
        )
        self.assertBankHolidayName(
            name,
            "1982-11-01",
            "1983-10-31",
            "1999-11-01",
            "2000-11-06",
            "2001-11-05",
        )
        self.assertBankHolidayName(name, range(1938, self.end_year))
        self.assertNoBankHolidayName(name, range(1933, 1938))

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "Año Nuevo"),
            ("2022-01-06", "Día de los Niños"),
            ("2022-02-28", "Carnaval"),
            ("2022-03-01", "Carnaval"),
            ("2022-04-11", "Semana de Turismo"),
            ("2022-04-12", "Semana de Turismo"),
            ("2022-04-13", "Semana de Turismo"),
            ("2022-04-14", "Semana de Turismo"),
            ("2022-04-15", "Semana de Turismo"),
            ("2022-04-18", "Desembarco de los 33 Orientales"),
            ("2022-05-01", "Día de los Trabajadores"),
            ("2022-05-16", "Batalla de Las Piedras"),
            ("2022-06-19", "Natalicio de Artigas"),
            ("2022-07-18", "Jura de la Constitución"),
            ("2022-08-25", "Declaratoria de la Independencia"),
            ("2022-10-10", "Día de la Diversidad Cultural"),
            ("2022-11-02", "Día de los Difuntos"),
            ("2022-12-25", "Día de la Familia"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-01-06", "Children's Day"),
            ("2022-02-28", "Carnival"),
            ("2022-03-01", "Carnival"),
            ("2022-04-11", "Tourism Week"),
            ("2022-04-12", "Tourism Week"),
            ("2022-04-13", "Tourism Week"),
            ("2022-04-14", "Tourism Week"),
            ("2022-04-15", "Tourism Week"),
            ("2022-04-18", "Landing of the 33 Patriots"),
            ("2022-05-01", "Workers' Day"),
            ("2022-05-16", "Battle of Las Piedras"),
            ("2022-06-19", "Birthday of Artigas"),
            ("2022-07-18", "Constitution Day"),
            ("2022-08-25", "Independence Day"),
            ("2022-10-10", "Cultural Diversity Day"),
            ("2022-11-02", "All Souls' Day"),
            ("2022-12-25", "Day of the Family"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2022-01-01", "Новий рік"),
            ("2022-01-06", "День дітей"),
            ("2022-02-28", "Карнавал"),
            ("2022-03-01", "Карнавал"),
            ("2022-04-11", "Тиждень туризму"),
            ("2022-04-12", "Тиждень туризму"),
            ("2022-04-13", "Тиждень туризму"),
            ("2022-04-14", "Тиждень туризму"),
            ("2022-04-15", "Тиждень туризму"),
            ("2022-04-18", "День висадки 33 патріотів"),
            ("2022-05-01", "День трудящих"),
            ("2022-05-16", "День битви при Лас-Пʼєдрас"),
            ("2022-06-19", "Річниця Артігаса"),
            ("2022-07-18", "День присяги Конституції"),
            ("2022-08-25", "День проголошення незалежності"),
            ("2022-10-10", "День культурного різноманіття"),
            ("2022-11-02", "День усіх померлих"),
            ("2022-12-25", "День родини"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.cayman_islands import CaymanIslands
from tests.common import CommonCountryTests


class TestCaymanIslands(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(CaymanIslands)

    def test_special_holidays(self):
        self.assertHoliday(
            "2009-11-06",
            "2011-04-29",
            "2012-06-04",
            "2022-09-19",
            "2023-05-08",
        )
        self.assertHolidayName(
            "Referendum Day",
            "2012-07-18",
            "2019-12-19",
        )
        self.assertHolidayName(
            "General Election Day",
            "2009-05-20",
            "2013-05-22",
            "2017-05-24",
            "2021-04-14",
            "2025-04-30",
        )

    def test_new_years_day(self):
        name = "New Year's Day"
        self.assertHolidayName(name, (f"{year}-01-01" for year in self.full_range))
        obs_dts = (
            "2011-01-03",
            "2012-01-02",
            "2017-01-02",
            "2022-01-03",
            "2023-01-02",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_national_heroes_day(self):
        name = "National Heroes Day"
        self.assertHolidayName(
            name,
            "2020-01-27",
            "2021-01-25",
            "2022-01-24",
            "2023-01-23",
            "2024-01-22",
            "2025-01-27",
        )
        self.assertHolidayName(name, self.full_range)

    def test_ash_wednesday(self):
        name = "Ash Wednesday"
        self.assertHolidayName(
            name,
            "2020-02-26",
            "2021-02-17",
            "2022-03-02",
            "2023-02-22",
            "2024-02-14",
            "2025-03-05",
        )
        self.assertHolidayName(name, self.full_range)

    def test_good_friday(self):
        name = "Good Friday"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_monday(self):
        name = "Easter Monday"
        self.assertHolidayName(
            name,
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, self.full_range)

    def test_emancipation_day(self):
        name = "Emancipation Day"
        self.assertHolidayName(
            name,
            "2024-05-06",
            "2025-05-05",
            "2026-05-04",
            "2027-05-03",
            "2028-05-01",
        )
        self.assertHolidayName(name, range(2024, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 2024))

    def test_discovery_day(self):
        name = "Discovery Day"
        self.assertHolidayName(
            name,
            "2020-05-18",
            "2021-05-17",
            "2022-05-16",
            "2023-05-15",
            "2024-05-20",
            "2025-05-19",
        )
        self.assertHolidayName(name, self.full_range)

    def test_queens_birthday(self):
        name = "Queen's Birthday"
        self.assertHolidayName(
            name,
            "2018-06-11",
            "2019-06-10",
            "2020-06-15",
            "2021-06-14",
            "2022-06-06",
        )
        self.assertHolidayName(name, range(self.start_year, 2023))
        self.assertNoHolidayName(name, range(2023, self.end_year))

    def test_kings_birthday(self):
        name = "King's Birthday"
        self.assertHolidayName(
            name,
            "2023-06-19",
            "2024-06-17",
            "2025-06-23",
        )
        self.assertHolidayName(name, range(2023, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 2023))

    def test_constitution_day(self):
        name = "Constitution Day"
        self.assertHolidayName(
            name,
            "2020-07-06",
            "2021-07-05",
            "2022-07-04",
            "2023-07-03",
            "2024-07-01",
            "2025-07-07",
        )
        self.assertHolidayName(name, self.full_range)

    def test_remembrance_day(self):
        name = "Remembrance Day"
        self.assertHolidayName(
            name,
            "2020-11-09",
            "2021-11-08",
            "2022-11-14",
            "2023-11-13",
            "2024-11-11",
            "2025-11-10",
        )
        self.assertHolidayName(name, self.full_range)

    def test_christmas_day(self):
        name = "Christmas Day"
        self.assertHolidayName(name, (f"{year}-12-25" for year in self.full_range))
        obs_dts = (
            "2010-12-27",
            "2011-12-27",
            "2016-12-27",
            "2021-12-27",
            "2022-12-27",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_boxing_day(self):
        name = "Boxing Day"
        self.assertHolidayName(name, (f"{year}-12-26" for year in self.full_range))
        obs_dts = (
            "2010-12-28",
            "2015-12-28",
            "2020-12-28",
            "2021-12-28",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2025-01-01", "New Year's Day"),
            ("2025-01-27", "National Heroes Day"),
            ("2025-03-05", "Ash Wednesday"),
            ("2025-04-18", "Good Friday"),
            ("2025-04-21", "Easter Monday"),
            ("2025-04-30", "General Election Day"),
            ("2025-05-05", "Emancipation Day"),
            ("2025-05-19", "Discovery Day"),
            ("2025-06-23", "King's Birthday"),
            ("2025-07-07", "Constitution Day"),
            ("2025-11-10", "Remembrance Day"),
            ("2025-12-25", "Christmas Day"),
            ("2025-12-26", "Boxing Day"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2025-01-01", "New Year's Day"),
            ("2025-01-27", "National Heroes Day"),
            ("2025-03-05", "Ash Wednesday"),
            ("2025-04-18", "Good Friday"),
            ("2025-04-21", "Easter Monday"),
            ("2025-04-30", "General Election Day"),
            ("2025-05-05", "Emancipation Day"),
            ("2025-05-19", "Discovery Day"),
            ("2025-06-23", "King's Birthday"),
            ("2025-07-07", "Constitution Day"),
            ("2025-11-10", "Remembrance Day"),
            ("2025-12-25", "Christmas Day"),
            ("2025-12-26", "Boxing Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.cuba import Cuba
from tests.common import CommonCountryTests


class TestCuba(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Cuba)

    def test_liberation_day(self):
        name = "Triunfo de la Revolución"
        name_observed = f"{name} (observado)"
        self.assertHolidayName(name, (f"{year}-01-01" for year in self.full_range))
        obs_dts = (
            "1989-01-02",
            "1995-01-02",
            "2006-01-02",
        )
        self.assertHolidayName(name_observed, obs_dts)
        self.assertNoHolidayName(name_observed, range(2014, self.end_year))
        self.assertNoNonObservedHoliday(obs_dts)

    def test_victory_day(self):
        name = "Día de la Victoria"
        self.assertHolidayName(name, (f"{year}-01-02" for year in range(2008, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2008))

    def test_good_friday(self):
        name = "Viernes Santo"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, range(2012, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 2012))

    def test_christmas_day(self):
        name = "Día de Navidad"
        self.assertHolidayName(
            name,
            (
                f"{year}-12-25"
                for year in (*range(self.start_year, 1969), *range(1997, self.end_year))
            ),
        )
        self.assertNoHolidayName(name, range(1969, 1997))

    def test_new_years_eve(self):
        name = "Fiesta de Fin de Año"
        self.assertHolidayName(name, (f"{year}-12-31" for year in range(2007, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2007))

    def test_international_workers_day(self):
        name = "Día Internacional de los Trabajadores"
        self.assertHolidayName(name, (f"{year}-05-01" for year in self.full_range))
        obs_dts = (
            "2011-05-02",
            "2016-05-02",
            "2022-05-02",
        )
        self.assertHolidayName(f"{name} (observado)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_assault_moncada_day(self):
        self.assertHolidayName(
            "Conmemoración del asalto a Moncada",
            (f"{year}-07-25" for year in self.full_range),
            (f"{year}-07-27" for year in self.full_range),
        )

    def test_national_rebellion_day(self):
        self.assertHolidayName(
            "Día de la Rebeldía Nacional", (f"{year}-07-26" for year in self.full_range)
        )

    def test_independence_day(self):
        name = "Inicio de las Guerras de Independencia"
        self.assertHolidayName(name, (f"{year}-10-10" for year in self.full_range))
        obs_dts = (
            "2004-10-11",
            "2010-10-11",
            "2021-10-11",
        )
        self.assertHolidayName(f"{name} (observado)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_2018(self):
        # https://www.officeholidays.com/countries/cuba/2018
        self.assertHolidayDatesInYear(
            2018,
            "2018-01-01",
            "2018-01-02",
            "2018-03-30",
            "2018-05-01",
            "2018-07-25",
            "2018-07-26",
            "2018-07-27",
            "2018-10-10",
            "2018-12-25",
            "2018-12-31",
        )

    def test_2019(self):
        # https://www.officeholidays.com/countries/cuba/2019
        self.assertHolidayDatesInYear(
            2019,
            "2019-01-01",
            "2019-01-02",
            "2019-04-19",
            "2019-05-01",
            "2019-07-25",
            "2019-07-26",
            "2019-07-27",
            "2019-10-10",
            "2019-12-25",
            "2019-12-31",
        )

    def test_2020(self):
        # https://www.officeholidays.com/countries/cuba/2020
        self.assertHolidayDatesInYear(
            2020,
            "2020-01-01",
            "2020-01-02",
            "2020-04-10",
            "2020-05-01",
            "2020-07-25",
            "2020-07-26",
            "2020-07-27",
            "2020-10-10",
            "2020-12-25",
            "2020-12-31",
        )

    def test_2021(self):
        # https://www.officeholidays.com/countries/cuba/2021
        self.assertHolidayDatesInYear(
            2021,
            "2021-01-01",
            "2021-01-02",
            "2021-04-02",
            "2021-05-01",
            "2021-07-25",
            "2021-07-26",
            "2021-07-27",
            "2021-10-10",
            "2021-10-11",
            "2021-12-25",
            "2021-12-31",
        )

    def test_2022(self):
        # https://www.officeholidays.com/countries/cuba/2022
        self.assertHolidayDatesInYear(
            2022,
            "2022-01-01",
            "2022-01-02",
            "2022-04-15",
            "2022-05-01",
            "2022-05-02",
            "2022-07-25",
            "2022-07-26",
            "2022-07-27",
            "2022-10-10",
            "2022-12-25",
            "2022-12-31",
        )

    def test_2023(self):
        # https://www.officeholidays.com/countries/cuba/2023
        self.assertHolidayDatesInYear(
            2023,
            "2023-01-01",
            "2023-01-02",
            "2023-04-07",
            "2023-05-01",
            "2023-07-25",
            "2023-07-26",
            "2023-07-27",
            "2023-10-10",
            "2023-12-25",
            "2023-12-31",
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "Triunfo de la Revolución"),
            ("2022-01-02", "Día de la Victoria"),
            ("2022-04-15", "Viernes Santo"),
            ("2022-05-01", "Día Internacional de los Trabajadores"),
            ("2022-05-02", "Día Internacional de los Trabajadores (observado)"),
            ("2022-07-25", "Conmemoración del asalto a Moncada"),
            ("2022-07-26", "Día de la Rebeldía Nacional"),
            ("2022-07-27", "Conmemoración del asalto a Moncada"),
            ("2022-10-10", "Inicio de las Guerras de Independencia"),
            ("2022-12-25", "Día de Navidad"),
            ("2022-12-31", "Fiesta de Fin de Año"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "Liberation Day"),
            ("2022-01-02", "Victory Day"),
            ("2022-04-15", "Good Friday"),
            ("2022-05-01", "International Workers' Day"),
            ("2022-05-02", "International Workers' Day (observed)"),
            ("2022-07-25", "Commemoration of the Assault of the Moncada garrison"),
            ("2022-07-26", "Day of the National Rebellion"),
            ("2022-07-27", "Commemoration of the Assault of the Moncada garrison"),
            ("2022-10-10", "Independence Day"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-31", "New Year's Eve"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2022-01-01", "Тріумф революції"),
            ("2022-01-02", "День Перемоги"),
            ("2022-04-15", "Страсна пʼятниця"),
            ("2022-05-01", "Міжнародний день трудящих"),
            ("2022-05-02", "Міжнародний день трудящих (вихідний)"),
            ("2022-07-25", "Вшанування памʼяті штурму Монкади"),
            ("2022-07-26", "День національного повстання"),
            ("2022-07-27", "Вшанування памʼяті штурму Монкади"),
            ("2022-10-10", "Початок війни за незалежність"),
            ("2022-12-25", "Різдво Христове"),
            ("2022-12-31", "Переддень Нового року"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from datetime import date
from unittest import TestCase

from holidays.calendars.gregorian import (
    JAN,
    MAR,
    MAY,
    JUN,
    JUL,
    NOV,
    DEC,
    WED,
    THU,
    _timedelta,
)
from holidays.constants import HALF_DAY
from holidays.financial.ny_stock_exchange import NewYorkStockExchange
from tests.common import CommonFinancialTests


class TestNewYorkStockExchange(CommonFinancialTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(NewYorkStockExchange)

    def test_no_holidays(self):
        super().test_no_holidays()

        self.assertNoHolidays(
            NewYorkStockExchange(categories=HALF_DAY, years=range(self.start_year, 1908))
        )

    def test_new_years_day(self):
        name = "New Year's Day"
        name_observed = f"{name} (observed)"
        self.assertNonObservedHolidayName(name, (f"{year}-01-01" for year in self.full_range))
        obs_dts = (
            "2012-01-02",
            "2017-01-02",
            "2023-01-02",
        )
        self.assertHolidayName(name_observed, obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)
        no_obs_dts = (
            "1943-12-31",
            "1948-12-31",
            "1993-12-31",
            "1999-12-31",
            "2004-12-31",
            "2010-12-31",
            "2021-12-31",
        )
        self.assertNoHolidayName(name_observed, no_obs_dts)
        self.assertNoHoliday(no_obs_dts)

    def test_martin_luther_king_jr_day(self):
        name = "Martin Luther King Jr. Day"
        self.assertHolidayName(
            name,
            "2020-01-20",
            "2021-01-18",
            "2022-01-17",
            "2023-01-16",
            "2024-01-15",
            "2025-01-20",
        )
        self.assertHolidayName(name, range(1998, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 1998))

    def test_lincolns_birthday(self):
        name = "Lincoln's Birthday"
        name_observed = f"{name} (observed)"
        self.assertNonObservedHolidayName(name, (f"{year}-02-12" for year in range(1896, 1954)))
        self.assertNoHolidayName(
            name, range(self.start_year, 1896), range(1954, 1968), range(1969, self.end_year)
        )
        obs_dts = (
            "1939-02-13",
            "1950-02-13",
        )
        self.assertHolidayName(name_observed, obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)
        no_obs_dts = (
            "1944-02-11",
            "1949-02-11",
        )
        self.assertNoHolidayName(name_observed, no_obs_dts)
        self.assertNoHoliday(no_obs_dts)

    def test_washingtons_birthday(self):
        name = "Washington's Birthday"
        name_observed = f"{name} (observed)"
        self.assertNonObservedHolidayName(
            name, (f"{year}-02-22" for year in range(self.start_year, 1971))
        )
        self.assertHolidayName(
            name,
            "2020-02-17",
            "2021-02-15",
            "2022-02-21",
            "2023-02-20",
            "2024-02-19",
            "2025-02-17",
        )
        self.assertHolidayName(name, range(1971, self.end_year))
        obs_dts = (
            "1964-02-21",
            "1969-02-21",
            "1970-02-23",
        )
        self.assertHolidayName(name_observed, obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)
        no_obs_dts = (
            "1941-02-21",
            "1947-02-21",
        )
        self.assertNoHolidayName(name_observed, no_obs_dts)
        self.assertNoHoliday(no_obs_dts)

    def test_good_friday(self):
        name = "Good Friday"
        years_absent = {1898, 1906, 1907}
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(
            name, (year for year in self.full_range if year not in years_absent)
        )
        self.assertNoHolidayName(name, years_absent)

    def test_memorial_day(self):
        name = "Memorial Day"
        name_observed = f"{name} (observed)"
        self.assertNonObservedHolidayName(name, (f"{year}-05-30" for year in range(1873, 1971)))
        self.assertHolidayName(
            name,
            "2020-05-25",
            "2021-05-31",
            "2022-05-30",
            "2023-05-29",
            "2024-05-27",
            "2025-05-26",
        )
        self.assertHolidayName(name, range(1971, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 1873))
        obs_dts = (
            "1964-05-29",
            "1965-05-31",
            "1970-05-29",
        )
        self.assertHolidayName(name_observed, obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)
        no_obs_dts = (
            "1936-05-29",
            "1942-05-29",
        )
        self.assertNoHolidayName(name_observed, no_obs_dts)
        self.assertNoHoliday(no_obs_dts)

    def test_flag_day(self):
        name = "Flag Day"
        name_observed = f"{name} (observed)"
        self.assertNonObservedHolidayName(name, (f"{year}-06-14" for year in range(1916, 1954)))
        self.assertNoHolidayName(name, range(self.start_year, 1916), range(1954, self.end_year))
        obs_dts = (
            "1936-06-15",
            "1942-06-15",
            "1953-06-15",
        )
        self.assertHolidayName(name_observed, obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)
        no_obs_dts = (
            "1947-06-13",
            "1952-06-13",
        )
        self.assertNoHolidayName(name_observed, no_obs_dts)
        self.assertNoHoliday(no_obs_dts)

    def test_juneteenth_national_independence_day(self):
        name = "Juneteenth National Independence Day"
        self.assertNonObservedHolidayName(
            name, (f"{year}-06-19" for year in range(2022, self.end_year))
        )
        self.assertNoHolidayName(name, range(self.start_year, 2022))
        obs_dts = (
            "2022-06-20",
            "2027-06-18",
            "2032-06-18",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_independence_day(self):
        name = "Independence Day"
        name_observed = f"{name} (observed)"
        self.assertNonObservedHolidayName(name, (f"{year}-07-04" for year in self.full_range))
        obs_dts = (
            "2015-07-03",
            "2020-07-03",
            "2021-07-05",
        )
        self.assertHolidayName(name_observed, obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)
        no_obs_dts = (
            "1936-07-03",
            "1942-07-03",
        )
        self.assertNoHolidayName(name_observed, no_obs_dts)
        self.assertNoHoliday(no_obs_dts)

    def test_labor_day(self):
        name = "Labor Day"
        self.assertHolidayName(
            name,
            "2020-09-07",
            "2021-09-06",
            "2022-09-05",
            "2023-09-04",
            "2024-09-02",
            "2025-09-01",
        )
        self.assertHolidayName(name, range(1887, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 1887))

    def test_columbus_day(self):
        name = "Columbus Day"
        name_observed = f"{name} (observed)"
        self.assertNonObservedHolidayName(name, (f"{year}-10-12" for year in range(1909, 1954)))
        self.assertNoHolidayName(name, range(self.start_year, 1909), range(1954, self.end_year))
        obs_dts = (
            "1941-10-13",
            "1947-10-13",
            "1952-10-13",
        )
        self.assertHolidayName(name_observed, obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)
        no_obs_dts = (
            "1940-10-11",
            "1946-10-11",
        )
        self.assertNoHolidayName(name_observed, no_obs_dts)
        self.assertNoHoliday(no_obs_dts)

    def test_election_day(self):
        name = "Election Day"
        self.assertHolidayName(
            name,
            "1950-11-07",
            "1968-11-05",
            "1972-11-07",
            "1976-11-02",
            "1980-11-04",
        )
        self.assertHolidayName(name, range(self.start_year, 1969))
        self.assertNoHolidayName(
            name,
            range(1969, 1972),
            range(1973, 1976),
            range(1977, 1980),
            range(1981, self.end_year),
        )

    def test_veterans_day(self):
        name = "Veteran's Day"
        name_observed = f"{name} (observed)"
        self.assertNonObservedHolidayName(name, (f"{year}-11-11" for year in range(1934, 1954)))
        self.assertNoHolidayName(
            name,
            range(self.start_year, 1918),
            range(1919, 1921),
            range(1922, 1934),
            range(1954, self.end_year),
        )
        obs_dts = (
            "1945-11-12",
            "1951-11-12",
        )
        self.assertHolidayName(name_observed, obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)
        no_obs_dts = (
            "1944-11-10",
            "1950-11-10",
        )
        self.assertNoHolidayName(name_observed, no_obs_dts)
        self.assertNoHoliday(no_obs_dts)

    def test_thanksgiving_day(self):
        name = "Thanksgiving Day"
        self.assertHolidayName(
            name,
            # Last Thu of Nov and other exceptions.
            "1863-11-26",
            "1864-11-24",
            "1865-12-07",
            "1866-11-29",
            "1867-11-28",
            "1868-11-26",
            "1869-11-18",
            # Last Thu of Nov.
            "1933-11-30",
            "1934-11-29",
            "1935-11-28",
            "1936-11-26",
            "1937-11-25",
            "1938-11-24",
            # Franksgiving.
            "1939-11-23",
            "1940-11-21",
            "1941-11-20",
            # 4th Thu of Nov.
            "2020-11-26",
            "2021-11-25",
            "2022-11-24",
            "2023-11-23",
            "2024-11-28",
            "2025-11-27",
        )
        self.assertHolidayName(name, self.full_range)

    def test_christmas_day(self):
        name = "Christmas Day"
        name_observed = f"{name} (observed)"
        self.assertNonObservedHolidayName(name, (f"{year}-12-25" for year in self.full_range))
        obs_dts = (
            "2016-12-26",
            "2021-12-24",
            "2022-12-26",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)
        no_obs_dts = (
            "1943-12-24",
            "1948-12-24",
        )
        self.assertNoHolidayName(name_observed, no_obs_dts)
        self.assertNoHoliday(no_obs_dts)

    def _make_special_holiday_list(self, begin, end, *, days=None):
        return [
            day
            for day in (_timedelta(begin, n) for n in range((end - begin).days + 1))
            if self.holidays._is_weekday(day) and (days is None or day.weekday() in days)
        ]

    def test_special_public_holidays(self):
        # NOTE: update the list whenever new historical holidays are added.
        special_holidays = [
            "1885-08-08",  # Funeral of former President Ulysses S. Grant.
            "1887-07-02",  # Saturday before Independence Day.
            "1887-12-24",  # Saturday before Christmas.
            "1888-03-12",  # Blizzard of 1888.
            "1888-03-13",  # Blizzard of 1888.
            "1888-09-01",  # Saturday before Labor Day.
            "1888-11-30",  # Friday after Thanksgiving Day.
            "1889-04-29",  # Centennial of George Washington's Inauguration.
            "1889-04-30",  # Centennial of George Washington's Inauguration.
            "1889-05-01",  # Centennial of George Washington's Inauguration.
            "1890-07-05",  # Saturday after Independence Day.
            "1891-12-26",  # Saturday after Christmas Day.
            "1892-07-02",  # Saturday before Independence Day.
            "1892-10-12",  # Columbian Celebration.
            "1892-10-21",  # Columbian Celebration.
            "1892-10-22",  # Columbian Celebration.
            "1893-04-27",  # Columbian Celebration.
            "1896-12-26",  # Saturday after Christmas Day.
            "1897-04-27",  # Grant's Birthday.
            "1898-05-04",  # Charter Day.
            "1898-07-02",  # Saturday before Independence Day.
            "1898-08-20",  # Welcome of naval commanders.
            "1898-09-03",  # Saturday before Labor Day.
            "1898-12-24",  # Saturday before Christmas.
            "1899-02-11",  # Saturday before Lincoln's Birthday.
            "1899-05-29",  # Monday before Decoration Day.
            "1899-07-03",  # Monday before Independence Day.
            "1899-09-29",  # Admiral Dewey Celebration.
            "1899-09-30",  # Admiral Dewey Celebration.
            "1899-11-25",  # Funeral of Vice-President Garret A. Hobart.
            "1900-04-14",  # Saturday after Good Friday.
            "1900-09-01",  # Saturday before Labor Day.
            "1900-12-24",  # Saturday before Christmas.
            "1901-02-02",  # Funeral of Queen Victoria of England.
            "1901-02-23",  # Saturday after Washington's Birthday.
            "1901-04-06",  # Saturday after Good Friday.
            "1901-04-27",  # Moved to temporary quarters in Produce Exchange.
            "1901-05-11",  # Enlarged temporary quarters in Produce Exchange.
            "1901-07-05",  # Friday after Independence Day.
            "1901-07-06",  # Saturday after Independence Day.
            "1901-08-31",  # Saturday before Labor Day.
            "1901-09-14",  # Death of President William McKinley.
            "1901-09-19",  # Funeral of President William McKinley.
            "1902-03-29",  # Saturday after Good Friday.
            "1902-05-31",  # Saturday after Decoration Day.
            "1902-07-05",  # Saturday after Independence Day.
            "1902-08-09",  # Coronation of King Edward VII of England.
            "1902-08-30",  # Saturday before Labor Day.
            "1903-02-21",  # Saturday before Washington's Birthday.
            "1903-04-11",  # Saturday after Good Friday.
            "1903-04-22",  # Opening of new NYSE building.
            "1903-09-05",  # Saturday before Labor Day.
            "1903-12-26",  # Saturday after Christmas Day.
            "1904-05-28",  # Saturday before Decoration Day.
            "1904-07-02",  # Saturday before Independence Day.
            "1904-09-03",  # Saturday before Labor Day.
            "1904-12-24",  # Saturday before Christmas.
            "1905-04-22",  # Saturday after Good Friday.
            "1907-02-23",  # Saturday after Washington's Birthday.
            "1907-03-30",  # Saturday after Good Friday.
            "1907-08-31",  # Saturday before Labor Day.
            "1908-04-18",  # Saturday after Good Friday.
            "1908-09-05",  # Saturday before Labor Day.
            "1908-12-26",  # Saturday after Christmas Day.
            "1909-02-13",  # Saturday after Lincoln's Birthday.
            "1909-04-10",  # Saturday after Good Friday.
            "1909-05-29",  # Saturday before Decoration Day.
            "1909-07-03",  # Saturday before Independence Day.
            "1909-09-04",  # Saturday before Labor Day.
            "1909-09-25",  # Reception Day of the Hudson-Fulton Celebration.
            "1910-03-26",  # Saturday after Good Friday.
            "1910-05-28",  # Saturday before Decoration Day.
            "1910-07-02",  # Saturday before Independence Day.
            "1910-09-03",  # Saturday before Labor Day.
            "1910-12-24",  # Saturday before Christmas.
            "1911-04-15",  # Saturday after Good Friday.
            "1911-09-02",  # Saturday before Labor Day.
            "1911-12-23",  # Saturday before Christmas.
            "1912-08-31",  # Saturday before Labor Day.
            "1912-11-02",  # Funeral of Vice-President James S. Sherman.
            "1913-03-22",  # Saturday after Good Friday.
            "1913-05-31",  # Saturday after Decoration Day.
            "1913-07-05",  # Saturday after Independence Day.
            "1913-08-30",  # Saturday before Labor Day.
            "1916-12-30",  # Saturday before New Year's Day.
            "1917-06-05",  # Draft Registration Day.
            "1917-08-04",  # Heat.
            "1917-09-01",  # Saturday before Labor Day.
            "1917-10-13",  # Saturday after Columbus Day.
            "1918-01-28",  # Heatless Day.
            "1918-02-04",  # Heatless Day.
            "1918-02-11",  # Heatless Day.
            "1918-09-12",  # Draft Registration Day.
            "1918-11-11",  # Armistice signed.
            "1919-03-25",  # Homecoming of 27th Division.
            "1919-05-06",  # Parade of 77th Division.
            "1919-05-31",  # Saturday after Decoration Day.
            "1919-07-05",  # Saturday after Independence Day.
            "1919-07-19",  # Heat and to allow offices to catch up on work.
            "1919-08-02",  # To allow offices to catch up on work.
            "1919-08-16",  # To allow offices to catch up on work.
            "1919-08-30",  # Saturday before Labor Day.
            "1919-09-10",  # Return of General John J. Pershing.
            "1920-04-03",  # Saturday after Good Friday.
            "1920-05-01",  # Many firms changed office locations.
            "1920-07-03",  # Saturday before Independence Day.
            "1920-09-04",  # Saturday before Labor Day.
            "1921-05-28",  # Saturday before Decoration Day.
            "1921-07-02",  # Saturday before Independence Day.
            "1921-09-03",  # Saturday before Labor Day.
            "1921-11-11",  # Veteran's Day.
            "1922-12-23",  # Saturday before Christmas.
            "1923-08-03",  # Death of President Warren G. Harding.
            "1923-08-10",  # Funeral of President Warren G. Harding.
            "1924-05-31",  # Saturday after Decoration Day.
            "1925-12-26",  # Saturday after Christmas Day.
            "1926-05-29",  # Saturday before Decoration Day.
            "1926-07-03",  # Saturday before Independence Day.
            "1926-09-04",  # Saturday before Labor Day.
            "1927-06-13",  # Parade for Colonel Charles A. Lindbergh.
            "1928-04-07",  # Heavy volume. To allow member firm offices to catch up on work.
            "1928-04-21",  # Heavy volume. To allow member firm offices to catch up on work.
            "1928-05-05",  # Heavy volume. To allow member firm offices to catch up on work.
            "1928-05-12",  # Heavy volume. To allow member firm offices to catch up on work.
            "1928-05-19",  # Heavy volume. To allow member firm offices to catch up on work.
            "1928-05-26",  # Heavy volume. To allow member firm offices to catch up on work.
            "1928-11-24",  # Heavy volume. To allow member firm offices to catch up on work.
            "1929-02-09",  # Heavy volume. To allow member firm offices to catch up on work.
            "1929-02-23",  # Saturday after Washington's Birthday.
            "1929-03-30",  # Saturday after Good Friday.
            "1929-08-31",  # Saturday before Labor Day.
            "1929-11-01",  # Catch Up Day.
            "1929-11-02",  # Catch Up Day.
            "1929-11-09",  # Catch Up Day.
            "1929-11-16",  # Catch Up Day.
            "1929-11-23",  # Catch Up Day.
            "1929-11-29",  # Catch Up Day.
            "1929-11-30",  # Catch Up Day.
            "1930-04-19",  # Saturday after Good Friday.
            "1930-05-31",  # Saturday after Decoration Day.
            "1930-07-05",  # Saturday after Independence Day.
            "1930-08-30",  # Saturday before Labor Day.
            "1931-09-05",  # Saturday before Labor Day.
            "1931-12-26",  # Saturday after Christmas Day.
            "1932-07-02",  # Saturday before Independence Day.
            "1933-01-07",  # Funeral of former President Calvin Coolidge.
            "1933-03-04",  # State Banking Holiday.
            "1933-03-06",  # National Banking Holiday.
            "1933-03-07",  # National Banking Holiday.
            "1933-03-08",  # National Banking Holiday.
            "1933-03-09",  # National Banking Holiday.
            "1933-03-10",  # National Banking Holiday.
            "1933-03-11",  # National Banking Holiday.
            "1933-03-13",  # National Banking Holiday.
            "1933-03-14",  # National Banking Holiday.
            "1933-07-28",  # Volume activity.
            "1933-08-05",  # Volume activity.
            "1933-08-12",  # Volume activity.
            "1933-08-19",  # Volume activity.
            "1933-08-26",  # Volume activity.
            "1933-09-02",  # Volume activity.
            "1936-12-26",  # Saturday after Christmas Day.
            "1937-05-29",  # Saturday before Decoration Day.
            "1937-07-03",  # Saturday before Independence Day.
            "1945-04-14",  # National Day of Mourning for President Franklin D. Roosevelt.
            "1945-08-15",  # V-J Day. End of World War II.
            "1945-08-16",  # V-J Day. End of World War II.
            "1945-10-13",  # Saturday after Columbus Day.
            "1945-10-27",  # Navy Day.
            "1945-12-24",  # Christmas Eve.
            "1946-02-23",  # Saturday after Washington's Birthday.
            "1946-05-25",  # Railroad strike.
            "1948-01-03",  # Severe weather conditions.
            "1949-12-24",  # Christmas Eve.
            "1950-12-23",  # Saturday before Christmas Eve.
            "1954-12-24",  # Christmas Eve.
            "1956-12-24",  # Christmas Eve.
            "1958-12-26",  # Day after Christmas.
            "1961-05-29",  # Day before Decoration Day.
            "1963-11-25",  # Funeral of President John F. Kennedy.
            "1965-12-24",  # Christmas Eve.
            "1968-02-12",  # Lincoln's Birthday.
            "1968-04-09",  # National Day of Mourning for Martin Luther King, Jr..
            "1968-07-05",  # Day after Independence Day.
            "1969-02-10",  # Heavy Snow.
            "1969-03-31",  # Funeral of former President Dwight D. Eisenhower.
            "1969-07-21",  # National Day of Participation for the Lunar Exploration.
            "1972-12-28",  # Funeral of former President Harry S. Truman.
            "1973-01-25",  # Funeral of former President Lyndon B. Johnson.
            "1977-07-14",  # Blackout in New York City.
            "1985-09-27",  # Hurricane Gloria.
            "1994-04-27",  # Funeral of former President Richard M. Nixon.
            "2001-09-11",  # Closed following Attacks on the World Trade Center.
            "2001-09-12",  # Closed following Attacks on the World Trade Center.
            "2001-09-13",  # Closed following Attacks on the World Trade Center.
            "2001-09-14",  # Closed following Attacks on the World Trade Center.
            "2004-06-11",  # National Day of Mourning for former President Ronald Reagan.
            "2007-01-02",  # National Day of Mourning for former President Gerald R. Ford.
            "2012-10-29",  # Hurricane Sandy.
            "2012-10-30",  # Hurricane Sandy.
            "2018-12-05",  # National Day of Mourning for former President George H. W. Bush.
            "2025-01-09",  # National Day of Mourning for former President Jimmy Carter.
        ]

        wwi_holidays = self._make_special_holiday_list(date(1914, JUL, 31), date(1914, NOV, 27))
        special_bank_holidays = self._make_special_holiday_list(
            date(1933, MAR, 6), date(1933, MAR, 14)
        )
        paperwork_crisis_holidays = self._make_special_holiday_list(
            date(1968, JUN, 12), date(1968, DEC, 31), days={WED}
        )
        self.assertHoliday(
            special_holidays,
            wwi_holidays,
            special_bank_holidays,
            paperwork_crisis_holidays,
        )

    def test_day_before_independence_day(self):
        name = "Day before Independence Day (markets close at 1:00pm)"
        self.assertNoHolidayName(name)
        self.assertHalfDayHolidayName(
            name,
            "1995-07-03",
            "1997-07-03",
            "2000-07-03",
            "2001-07-03",
            "2003-07-03",
            "2006-07-03",
            "2007-07-03",
            "2008-07-03",
            "2012-07-03",
            "2014-07-03",
            "2017-07-03",
            "2018-07-03",
            "2023-07-03",
            "2025-07-03",
        )
        self.assertNoHalfDayHolidayName(name, range(self.start_year, 1993), 1996, 2002)

    def test_day_after_thanksgiving(self):
        name = "Day after Thanksgiving Day (markets close at 1:00pm)"
        self.assertNoHolidayName(name)
        self.assertHalfDayHolidayName(
            name,
            "2020-11-27",
            "2021-11-26",
            "2022-11-25",
            "2023-11-24",
            "2024-11-29",
            "2025-11-28",
        )
        self.assertHalfDayHolidayName(name, range(1993, self.end_year))
        self.assertNoHalfDayHolidayName(name, range(self.start_year, 1993))

    def test_christmas_eve(self):
        name = "Christmas Eve (markets close at 1:00pm)"
        self.assertNoHolidayName(name)
        self.assertHalfDayHolidayName(
            name,
            "1996-12-24",
            "1997-12-24",
            "1998-12-24",
            "2001-12-24",
            "2002-12-24",
            "2003-12-24",
            "2007-12-24",
            "2008-12-24",
            "2009-12-24",
            "2012-12-24",
            "2013-12-24",
            "2014-12-24",
            "2015-12-24",
            "2018-12-24",
            "2019-12-24",
            "2020-12-24",
            "2024-12-24",
            "2025-12-24",
        )
        self.assertNoHalfDayHolidayName(name, range(self.start_year, 1951), range(1952, 1993))

    def test_special_half_day_holidays(self):
        special_holidays = [
            "1908-06-26",  # Funeral of former President Grover Cleveland (from 1:00pm).
            "1910-05-07",  # Death of King Edward VII of England (from 11:00am).
            "1917-08-29",  # Parade of National Guard (from 12:00pm).
            "1917-10-24",  # Liberty Day (from 12:00pm).
            "1918-04-26",  # Liberty Day (from 12:00pm).
            "1918-11-07",  # False armistice report (from 2:30pm).
            "1919-01-07",  # Funeral of former President Theodore Roosevelt (from 12:30pm).
            "1920-09-16",  # Wall Street explosion (from 12:00pm).
            "1924-02-06",  # Funeral of former President Woodrow Wilson (from 12:30pm).
            "1925-09-18",  # Funeral of former NYSE president Seymour L. Cromwell (from 2:30pm).
            "1928-05-21",  # Heavy volume. To allow member firm offices to catch up (from 2:00pm).
            "1928-05-22",  # Heavy volume. To allow member firm offices to catch up (from 2:00pm).
            "1928-05-23",  # Heavy volume. To allow member firm offices to catch up (from 2:00pm).
            "1928-05-24",  # Heavy volume. To allow member firm offices to catch up (from 2:00pm).
            "1928-05-25",  # Heavy volume. To allow member firm offices to catch up (from 2:00pm).
            "1929-11-06",  # Catch Up Day (from 1:00pm).
            "1929-11-07",  # Catch Up Day (from 1:00pm).
            "1929-11-08",  # Catch Up Day (from 1:00pm).
            "1929-11-11",  # Catch Up Day (from 1:00pm).
            "1929-11-12",  # Catch Up Day (from 1:00pm).
            "1929-11-13",  # Catch Up Day (from 1:00pm).
            "1929-11-14",  # Catch Up Day (from 1:00pm).
            "1929-11-15",  # Catch Up Day (from 1:00pm).
            "1929-11-18",  # Catch Up Day (from 1:00pm).
            "1929-11-19",  # Catch Up Day (from 1:00pm).
            "1929-11-20",  # Catch Up Day (from 1:00pm).
            "1929-11-21",  # Catch Up Day (from 1:00pm).
            "1929-11-22",  # Catch Up Day (from 1:00pm).
            "1930-03-11",  # Funeral of former President William Howard Taft (from 12:30pm).
            "1933-09-13",  # NRA demonstration (from 12:00pm).
            "1951-12-24",  # Christmas Eve (from 1:00pm).
            "1963-11-22",  # Assassination of President John F. Kennedy (from 2:07pm).
            "1964-10-23",  # Funeral of former President Herbert C. Hoover (from 2:00pm).
            "1966-01-06",  # Transit strike (from 2:00pm).
            "1966-01-07",  # Transit strike (from 2:00pm).
            "1966-01-10",  # Transit strike (from 2:00pm).
            "1966-01-11",  # Transit strike (from 2:00pm).
            "1966-01-12",  # Transit strike (from 2:00pm).
            "1966-01-13",  # Transit strike (from 2:00pm).
            "1966-01-14",  # Transit strike (from 2:00pm).
            "1967-08-08",  # Back office work load (from 2:00pm).
            "1967-08-09",  # Back office work load (from 2:00pm).
            "1967-08-10",  # Back office work load (from 2:00pm).
            "1967-08-11",  # Back office work load (from 2:00pm).
            "1967-08-14",  # Back office work load (from 2:00pm).
            "1967-08-15",  # Back office work load (from 2:00pm).
            "1967-08-16",  # Back office work load (from 2:00pm).
            "1967-08-17",  # Back office work load (from 2:00pm).
            "1967-08-18",  # Back office work load (from 2:00pm).
            "1974-12-24",  # Christmas Eve (from 2:00pm).
            "1975-02-12",  # Snowstorm (from 2:30pm).
            "1975-12-24",  # Christmas Eve (from 2:00pm).
            "1976-08-09",  # Hurricane watch (from 3:00pm).
            "1978-02-06",  # Snowstorm (from 2:00pm).
            "1981-03-30",  # Assassination attempt on President Reagan (from 3:17pm).
            "1981-09-09",  # Con Edison power failure in lower Manhattan (from 3:28pm).
            "1987-10-23",  # Shortened hours following market break (from 2:00pm).
            "1987-10-26",  # Shortened hours following market break (from 2:00pm).
            "1987-10-27",  # Shortened hours following market break (from 2:00pm).
            "1987-10-28",  # Shortened hours following market break (from 2:00pm).
            "1987-10-29",  # Shortened hours following market break (from 2:00pm).
            "1987-10-30",  # Shortened hours following market break (from 2:00pm).
            "1987-11-02",  # Shortened hours following market break (from 2:30pm).
            "1987-11-03",  # Shortened hours following market break (from 2:30pm).
            "1987-11-04",  # Shortened hours following market break (from 2:30pm).
            "1987-11-05",  # Shortened hours following market break (from 3:00pm).
            "1987-11-06",  # Shortened hours following market break (from 3:00pm).
            "1987-11-09",  # Shortened hours following market break (from 3:30pm).
            "1987-11-10",  # Shortened hours following market break (from 3:30pm).
            "1987-11-11",  # Shortened hours following market break (from 3:30pm).
            "1990-12-24",  # Christmas Eve (from 2:00pm).
            "1991-12-24",  # Christmas Eve (from 2:00pm).
            "1992-11-27",  # Day after Thanksgiving Day (from 2:00pm).
            "1992-12-24",  # Christmas Eve (from 2:00pm).
            "1994-02-11",  # Snowstorm (from 2:30pm).
            "1996-07-05",  # Day after Independence Day (from 1:00pm).
            "1997-12-26",  # Friday after Christmas Day (from 1:00pm).
            "1999-12-31",  # New Year's Eve (from 1:00pm).
            "2002-07-05",  # Day after Independence Day (from 1:00pm).
            "2003-12-26",  # Friday after Christmas Day (from 1:00pm).
            "2013-07-03",  # Day before Independence Day (from 1:00pm).
        ]

        back_office_holidays = self._make_special_holiday_list(
            date(1968, JAN, 22), date(1968, MAR, 1)
        )
        paperwork_crisis_holidays_1 = self._make_special_holiday_list(
            date(1969, JAN, 2), date(1969, JUL, 3), days={THU}
        )
        paperwork_crisis_holidays_2 = self._make_special_holiday_list(
            date(1969, JUL, 7), date(1969, DEC, 31)
        )
        paperwork_crisis_holidays_3 = self._make_special_holiday_list(
            date(1970, JAN, 2), date(1970, MAY, 1)
        )
        self.assertHalfDayHoliday(
            special_holidays,
            back_office_holidays,
            paperwork_crisis_holidays_1,
            paperwork_crisis_holidays_2,
            paperwork_crisis_holidays_3,
        )

    def test_2023(self):
        self.assertHolidaysInYear(
            2023,
            ("2023-01-02", "New Year's Day (observed)"),
            ("2023-01-16", "Martin Luther King Jr. Day"),
            ("2023-02-20", "Washington's Birthday"),
            ("2023-04-07", "Good Friday"),
            ("2023-05-29", "Memorial Day"),
            ("2023-06-19", "Juneteenth National Independence Day"),
            ("2023-07-04", "Independence Day"),
            ("2023-09-04", "Labor Day"),
            ("2023-11-23", "Thanksgiving Day"),
            ("2023-12-25", "Christmas Day"),
        )

    def test_half_day_2023(self):
        self.assertHalfDayHolidaysInYear(
            2023,
            ("2023-07-03", "Day before Independence Day (markets close at 1:00pm)"),
            ("2023-11-24", "Day after Thanksgiving Day (markets close at 1:00pm)"),
        )

    def test_2024(self):
        self.assertHolidaysInYear(
            2024,
            ("2024-01-01", "New Year's Day"),
            ("2024-01-15", "Martin Luther King Jr. Day"),
            ("2024-02-19", "Washington's Birthday"),
            ("2024-03-29", "Good Friday"),
            ("2024-05-27", "Memorial Day"),
            ("2024-06-19", "Juneteenth National Independence Day"),
            ("2024-07-04", "Independence Day"),
            ("2024-09-02", "Labor Day"),
            ("2024-11-28", "Thanksgiving Day"),
            ("2024-12-25", "Christmas Day"),
        )

    def test_half_day_2024(self):
        self.assertHalfDayHolidaysInYear(
            2024,
            ("2024-07-03", "Day before Independence Day (markets close at 1:00pm)"),
            ("2024-11-29", "Day after Thanksgiving Day (markets close at 1:00pm)"),
            ("2024-12-24", "Christmas Eve (markets close at 1:00pm)"),
        )

    def test_2025(self):
        self.assertHolidaysInYear(
            2025,
            ("2025-01-01", "New Year's Day"),
            ("2025-01-09", "National Day of Mourning for former President Jimmy Carter"),
            ("2025-01-20", "Martin Luther King Jr. Day"),
            ("2025-02-17", "Washington's Birthday"),
            ("2025-04-18", "Good Friday"),
            ("2025-05-26", "Memorial Day"),
            ("2025-06-19", "Juneteenth National Independence Day"),
            ("2025-07-04", "Independence Day"),
            ("2025-09-01", "Labor Day"),
            ("2025-11-27", "Thanksgiving Day"),
            ("2025-12-25", "Christmas Day"),
        )

    def test_half_day_2025(self):
        self.assertHalfDayHolidaysInYear(
            2025,
            ("2025-07-03", "Day before Independence Day (markets close at 1:00pm)"),
            ("2025-11-28", "Day after Thanksgiving Day (markets close at 1:00pm)"),
            ("2025-12-24", "Christmas Eve (markets close at 1:00pm)"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from __future__ import annotations

from gettext import gettext as tr
from typing import TYPE_CHECKING

from holidays.calendars import _CustomHinduHolidays, _CustomIslamicHolidays
from holidays.calendars.gregorian import MAR, APR, JUN, JUL, OCT, NOV
from holidays.constants import OPTIONAL, PUBLIC
from holidays.groups import (
    ChristianHolidays,
    HinduCalendarHolidays,
    InternationalHolidays,
    IslamicHolidays,
)
from holidays.observed_holiday_base import (
    ObservedHolidayBase,
    SUN_TO_NEXT_WORKDAY,
    SAT_SUN_TO_NEXT_WORKDAY,
    WORKDAY_TO_NEXT_WORKDAY,
)

if TYPE_CHECKING:
    from datetime import date


class TrinidadAndTobago(
    ObservedHolidayBase,
    ChristianHolidays,
    HinduCalendarHolidays,
    InternationalHolidays,
    IslamicHolidays,
):
    """Trinidad and Tobago holidays.

    References:
        * <https://web.archive.org/web/20250429121623/https://otp.tt/trinidad-and-tobago/national-holidays-and-awards/>
        * <https://web.archive.org/web/20250429120744/https://laws.gov.tt/ttdll-web2/revision/list?offset=380&q=&currentid=322>
        * <https://web.archive.org/web/20250429120707/https://www.guardian.co.tt/article-6.2.448640.3e9e366923>
        * <https://web.archive.org/web/20250429120548/https://www.guardian.co.tt/article-6.2.410311.3fe66fb00f>
        * <https://web.archive.org/web/20250418212357/https://www.nalis.gov.tt/resources/tt-content-guide/labour-day/>
        * <https://web.archive.org/web/20250429120701/https://www.facebook.com/groups/191766699268/posts/10160832951274269/>
        * <https://web.archive.org/web/20250429142054/https://www.facebook.com/plugins/post.php?href=https://www.facebook.com/100064996051675/posts/1101755938667598/>
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Trinidad_and_Tobago>
        * <https://en.wikipedia.org/wiki/Trinidad_and_Tobago_Carnival>
        * <https://en.wikipedia.org/wiki/Indian_Arrival_Day#Trinidad_and_Tobago>
        * <https://web.archive.org/web/20250429120402/https://www.timeanddate.com/holidays/trinidad/eid-al-fitr>
        * <https://web.archive.org/web/20250429120140/https://www.timeanddate.com/holidays/trinidad/diwali>
    """

    country = "TT"
    default_language = "en_TT"
    supported_categories = (OPTIONAL, PUBLIC)
    # %s (estimated).
    estimated_label = tr("%s (estimated)")
    # %s (observed).
    observed_label = tr("%s (observed)")
    # %s (observed, estimated).
    observed_estimated_label = tr("%s (observed, estimated)")
    supported_languages = ("en_TT", "en_US")
    # Trinidad and Tobago gained independence on August 31, 1962.
    start_year = 1963

    def __init__(self, *args, islamic_show_estimated: bool = True, **kwargs):
        """
        Args:
            islamic_show_estimated:
                Whether to add "estimated" label to Islamic holidays name
                if holiday date is estimated.
        """
        ChristianHolidays.__init__(self)
        HinduCalendarHolidays.__init__(self, cls=TrinidadAndTobagoHinduHolidays)
        InternationalHolidays.__init__(self)
        IslamicHolidays.__init__(
            self, cls=TrinidadAndTobagoIslamicHolidays, show_estimated=islamic_show_estimated
        )
        super().__init__(*args, **kwargs)

    def _populate_observed(self, dts: set[date], *, multiple: bool = False) -> None:
        for dt in sorted(dts):
            self._add_observed(
                dt,
                rule=WORKDAY_TO_NEXT_WORKDAY + SAT_SUN_TO_NEXT_WORKDAY
                if len(self.get_list(dt)) > 1
                else SUN_TO_NEXT_WORKDAY,
            )

    def _populate_public_holidays(self):
        dts_observed = set()

        # New Year's Day.
        dts_observed.add(self._add_new_years_day(tr("New Year's Day")))

        # Good Friday.
        self._add_good_friday(tr("Good Friday"))

        # Easter Monday.
        self._add_easter_monday(tr("Easter Monday"))

        if self._year >= 1996:
            # Spiritual Baptist Liberation Day.
            dts_observed.add(self._add_holiday_mar_30(tr("Spiritual Baptist Liberation Day")))

            # Indian Arrival Day.
            dts_observed.add(self._add_holiday_may_30(tr("Indian Arrival Day")))

        # Corpus Christi.
        dts_observed.add(self._add_corpus_christi_day(tr("Corpus Christi")))

        if self._year >= 1973:
            # Labor Day.
            dts_observed.add(self._add_holiday_jun_19(tr("Labour Day")))

        if self._year >= 1985:
            # African Emancipation Day.
            dts_observed.add(self._add_holiday_aug_1(tr("African Emancipation Day")))

        # Independence Day.
        dts_observed.add(self._add_holiday_aug_31(tr("Independence Day")))

        if self._year >= 1976:
            # Republic Day.
            dts_observed.add(self._add_holiday_sep_24(tr("Republic Day")))

        # Diwali.
        dts_observed.add(self._add_diwali(tr("Divali")))

        # Christmas Day.
        dts_observed.add(self._add_christmas_day(tr("Christmas Day")))

        # Boxing Day.
        dts_observed.add(self._add_christmas_day_two(tr("Boxing Day")))

        # Eid al-Fitr.
        dts_observed.update(self._add_eid_al_fitr_day(tr("Eid-Ul-Fitr")))

        if self.observed:
            self._populate_observed(dts_observed)

    def _populate_optional_holidays(self):
        # Carnival Monday.
        self._add_carnival_monday(tr("Carnival Monday"))

        # Carnival Tuesday.
        self._add_carnival_tuesday(tr("Carnival Tuesday"))


class TT(TrinidadAndTobago):
    pass


class TTO(TrinidadAndTobago):
    pass


class TrinidadAndTobagoHinduHolidays(_CustomHinduHolidays):
    DIWALI_DATES = {
        2012: (NOV, 13),
        2013: (NOV, 4),
        2014: (OCT, 23),
        2015: (NOV, 11),
        2016: (OCT, 29),
        2017: (OCT, 19),
        2018: (NOV, 7),
        2019: (OCT, 27),
        2020: (NOV, 14),
        2021: (NOV, 4),
        2022: (OCT, 24),
        2023: (NOV, 12),
        2024: (OCT, 31),
        2025: (OCT, 20),
    }


class TrinidadAndTobagoIslamicHolidays(_CustomIslamicHolidays):
    EID_AL_FITR_DATES_CONFIRMED_YEARS = (2012, 2025)
    EID_AL_FITR_DATES = {
        2014: (JUL, 29),
        2015: (JUL, 18),
        2017: (JUN, 26),
        2019: (JUN, 5),
        2023: (APR, 22),
        2025: (MAR, 31),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.financial.hong_kong_stock_exchange import HongKongStockExchange
from tests.common import CommonFinancialTests


class TestHongKongStockExchange(CommonFinancialTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(HongKongStockExchange)

    def test_code(self):
        self.assertTrue(hasattr(self.holidays, "market"))
        self.assertIsNone(getattr(self.holidays, "country", None))

    def test_2024(self):
        self.assertHolidaysInYear(
            2024,
            ("2024-01-01", "一月一日"),
            ("2024-02-12", "農曆年初三"),
            ("2024-02-13", "農曆年初四"),
            ("2024-03-29", "耶穌受難節"),
            ("2024-04-01", "復活節星期一"),
            ("2024-04-04", "清明節"),
            ("2024-05-01", "勞動節"),
            ("2024-05-15", "佛誕"),
            ("2024-06-10", "端午節"),
            ("2024-07-01", "香港特別行政區成立紀念日"),
            ("2024-09-18", "中秋節翌日"),
            ("2024-10-01", "國慶日"),
            ("2024-10-11", "重陽節"),
            ("2024-12-25", "聖誕節"),
            ("2024-12-26", "聖誕節後第一個周日"),
        )

    def test_2024_half_day(self):
        self.assertHalfDayHolidaysInYear(
            2024,
            ("2024-02-09", "農曆年初一的前一日（半日交易日）"),
            ("2024-12-24", "平安夜（半日交易日）"),
            ("2024-12-31", "新年前夕（半日交易日）"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2025-01-01", "一月一日"),
            ("2025-01-28", "農曆年初一的前一日（半日交易日）"),
            ("2025-01-29", "農曆年初一"),
            ("2025-01-30", "農曆年初二"),
            ("2025-01-31", "農曆年初三"),
            ("2025-04-04", "清明節"),
            ("2025-04-18", "耶穌受難節"),
            ("2025-04-21", "復活節星期一"),
            ("2025-05-01", "勞動節"),
            ("2025-05-05", "佛誕"),
            ("2025-07-01", "香港特別行政區成立紀念日"),
            ("2025-10-01", "國慶日"),
            ("2025-10-07", "中秋節翌日"),
            ("2025-10-29", "重陽節"),
            ("2025-12-24", "平安夜（半日交易日）"),
            ("2025-12-25", "聖誕節"),
            ("2025-12-26", "聖誕節後第一個周日"),
            ("2025-12-31", "新年前夕（半日交易日）"),
        )

    def test_l10n_en_hk(self):
        self.assertLocalizedHolidays(
            "en_HK",
            ("2025-01-01", "The first day of January"),
            ("2025-01-28", "The day preceding Lunar New Year's Day (Half-Day Trading Day)"),
            ("2025-01-29", "Lunar New Year's Day"),
            ("2025-01-30", "The second day of Lunar New Year"),
            ("2025-01-31", "The third day of Lunar New Year"),
            ("2025-04-04", "Ching Ming Festival"),
            ("2025-04-18", "Good Friday"),
            ("2025-04-21", "Easter Monday"),
            ("2025-05-01", "Labour Day"),
            ("2025-05-05", "The Birthday of the Buddha"),
            ("2025-07-01", "Hong Kong Special Administrative Region Establishment Day"),
            ("2025-10-01", "National Day"),
            ("2025-10-07", "The day following the Chinese Mid-Autumn Festival"),
            ("2025-10-29", "Chung Yeung Festival"),
            ("2025-12-24", "Christmas Eve (Half-Day Trading Day)"),
            ("2025-12-25", "Christmas Day"),
            ("2025-12-26", "The first weekday after Christmas Day"),
            ("2025-12-31", "New Year's Eve (Half-Day Trading Day)"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2025-01-01", "New Year's Day"),
            ("2025-01-28", "Chinese New Year's Eve (Half-Day Trading Day)"),
            ("2025-01-29", "Chinese New Year"),
            ("2025-01-30", "The second day of Chinese New Year"),
            ("2025-01-31", "The third day of Chinese New Year"),
            ("2025-04-04", "Tomb-Sweeping Day"),
            ("2025-04-18", "Good Friday"),
            ("2025-04-21", "Easter Monday"),
            ("2025-05-01", "Labor Day"),
            ("2025-05-05", "The Buddha's Birthday"),
            ("2025-07-01", "Hong Kong S.A.R. Establishment Day"),
            ("2025-10-01", "National Day"),
            ("2025-10-07", "The Day following Mid-Autumn Festival"),
            ("2025-10-29", "Double Ninth Festival"),
            ("2025-12-24", "Christmas Eve (Half-Day Trading Day)"),
            ("2025-12-25", "Christmas Day"),
            ("2025-12-26", "The first weekday after Christmas Day"),
            ("2025-12-31", "New Year's Eve (Half-Day Trading Day)"),
        )

    def test_l10n_th(self):
        self.assertLocalizedHolidays(
            "th",
            ("2025-01-01", "วันขึ้นปีใหม่"),
            ("2025-01-28", "วันก่อนวันตรุษจีน (วันซื้อขายครึ่งวัน)"),
            ("2025-01-29", "วันตรุษจีน"),
            ("2025-01-30", "วันตรุษจีนวันที่สอง"),
            ("2025-01-31", "วันตรุษจีนวันที่สาม"),
            ("2025-04-04", "วันเช็งเม้ง"),
            ("2025-04-18", "วันศุกร์ประเสริฐ"),
            ("2025-04-21", "วันจันทร์อีสเตอร์"),
            ("2025-05-01", "วันแรงงาน"),
            ("2025-05-05", "วันวิสาขบูชา"),
            ("2025-07-01", "วันสถาปนาเขตบริหารพิเศษฮ่องกง"),
            ("2025-10-01", "วันชาติจีน"),
            ("2025-10-07", "วันหลังวันไหว้พระจันทร์"),
            ("2025-10-29", "วันไหว้บรรพบุรุษ"),
            ("2025-12-24", "วันคริสต์มาสอีฟ (วันซื้อขายครึ่งวัน)"),
            ("2025-12-25", "วันคริสต์มาส"),
            ("2025-12-26", "วันหลังวันคริสต์มาส"),
            ("2025-12-31", "วันสิ้นปี (วันซื้อขายครึ่งวัน)"),
        )

    def test_l10n_zh_cn(self):
        self.assertLocalizedHolidays(
            "zh_CN",
            ("2025-01-01", "一月一日"),
            ("2025-01-28", "农历除夕（半日交易日）"),
            ("2025-01-29", "农历年初一"),
            ("2025-01-30", "农历年初二"),
            ("2025-01-31", "农历年初三"),
            ("2025-04-04", "清明节"),
            ("2025-04-18", "耶稣受难节"),
            ("2025-04-21", "复活节星期一"),
            ("2025-05-01", "劳动节"),
            ("2025-05-05", "佛诞"),
            ("2025-07-01", "香港特别行政区成立纪念日"),
            ("2025-10-01", "国庆日"),
            ("2025-10-07", "中秋节翌日"),
            ("2025-10-29", "重阳节"),
            ("2025-12-24", "平安夜（半日交易日）"),
            ("2025-12-25", "圣诞节"),
            ("2025-12-26", "圣诞节后第一个周日"),
            ("2025-12-31", "新年前夕（半日交易日）"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from holidays.calendars.balinese_saka import _BalineseSakaLunar
from holidays.calendars.buddhist import _BuddhistLunisolar, _CustomBuddhistHolidays
from holidays.calendars.burmese import _BurmeseLunisolar
from holidays.calendars.chinese import _ChineseLunisolar, _CustomChineseHolidays
from holidays.calendars.custom import _CustomCalendar
from holidays.calendars.gregorian import GREGORIAN_CALENDAR
from holidays.calendars.hebrew import _HebrewLunisolar
from holidays.calendars.hindu import _CustomHinduHolidays, _HinduLunisolar
from holidays.calendars.islamic import _CustomIslamicHolidays, _IslamicLunar
from holidays.calendars.julian import JULIAN_CALENDAR
from holidays.calendars.julian_revised import JULIAN_REVISED_CALENDAR
from holidays.calendars.mongolian import _CustomMongolianHolidays, _MongolianLunisolar
from holidays.calendars.persian import _Persian
from holidays.calendars.sinhala import _SinhalaLunar, _CustomSinhalaHolidays
from holidays.calendars.thai import _ThaiLunisolar, KHMER_CALENDAR, THAI_CALENDAR
from holidays.calendars.tibetan import _TibetanLunisolar, _CustomTibetanHolidays



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from holidays.calendars.ethiopian import ETHIOPIAN_CALENDAR
from holidays.constants import GOVERNMENT, PUBLIC
from holidays.groups import ChristianHolidays, InternationalHolidays, IslamicHolidays
from holidays.holiday_base import HolidayBase


class Eritrea(HolidayBase, ChristianHolidays, InternationalHolidays, IslamicHolidays):
    """Eritrea holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Eritrea>
        * <https://web.archive.org/web/20230313234629/https://explore-eritrea.com/working-hours-and-holidays/>
        * <https://web.archive.org/web/20110903130335/http://www.eritrea.be/old/eritrea-religions.htm>
        * <https://web.archive.org/web/20250820085307/https://www.mintageworld.com/media/detail/11411-fenkil-day-in-eritrea/>
    """

    country = "ER"
    # %s (estimated).
    estimated_label = "%s (estimated)"
    # On 28 May 1993, Eritrea was admitted into the United Nations as the 182nd member state.
    start_year = 1994
    supported_categories = (GOVERNMENT, PUBLIC)

    def __init__(self, *args, islamic_show_estimated: bool = True, **kwargs):
        """
        Args:
            islamic_show_estimated:
                Whether to add "estimated" label to Islamic holidays name
                if holiday date is estimated.
        """
        ChristianHolidays.__init__(self, ETHIOPIAN_CALENDAR)
        InternationalHolidays.__init__(self)
        IslamicHolidays.__init__(self, show_estimated=islamic_show_estimated)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_new_years_day("New Year's Day")

        # Orthodox Christmas.
        self._add_christmas_day("Orthodox Christmas")

        # Epiphany.
        self._add_epiphany_day("Epiphany")

        # Women's Day.
        self._add_womens_day("Women's Day")

        # Good Friday.
        self._add_good_friday("Good Friday")

        # Orthodox Easter.
        self._add_easter_sunday("Orthodox Easter")

        # International Workers' Day.
        self._add_labor_day("International Workers' Day")

        # Independence Day.
        self._add_holiday_may_24("Independence Day")

        # Martyrs' Day.
        self._add_holiday_jun_20("Martyrs' Day")

        # Revolution Day.
        self._add_holiday_sep_1("Revolution Day")

        # Ethiopian New Year.
        self._add_ethiopian_new_year("Ethiopian New Year")

        # Finding of the True Cross.
        self._add_finding_of_true_cross("Finding of the True Cross")

        # Prophet's Birthday.
        self._add_mawlid_day("Prophet's Birthday")

        # Eid al-Fitr.
        self._add_eid_al_fitr_day("Eid al-Fitr")

        # Eid al-Adha.
        self._add_eid_al_adha_day("Eid al-Adha")

    def _populate_government_holidays(self):
        # Fenkil Day.
        self._add_holiday_feb_10("Fenkil Day")


class ER(Eritrea):
    pass


class ERI(Eritrea):
    pass



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.nigeria import Nigeria
from tests.common import CommonCountryTests


class TestNigeria(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Nigeria)

    def test_special_holidays(self):
        self.assertHoliday(
            "2007-04-12",
            "2007-04-13",
            "2010-05-06",
            "2019-02-22",
            "2025-07-15",
        )

    def test_new_years_day(self):
        name = "New Year's Day"
        self.assertHolidayName(name, (f"{year}-01-01" for year in self.full_range))
        obs_dts = (
            "2017-01-02",
            "2022-01-03",
            "2023-01-02",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_good_friday(self):
        name = "Good Friday"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_monday(self):
        name = "Easter Monday"
        self.assertHolidayName(
            name,
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, self.full_range)

    def test_workers_day(self):
        name = "Workers' Day"
        self.assertHolidayName(name, (f"{year}-05-01" for year in range(1985, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1985))
        obs_dts = (
            "2016-05-02",
            "2021-05-03",
            "2022-05-04",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_democracy_day(self):
        name = "Democracy Day"
        self.assertHolidayName(
            name,
            (f"{year}-05-29" for year in range(2000, 2019)),
            (f"{year}-06-12" for year in range(2019, self.end_year)),
        )
        self.assertNoHolidayName(name, range(self.start_year, 2000))
        obs_dts = (
            "2016-05-30",
            "2021-06-14",
            "2022-06-13",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_presidential_inauguration_day(self):
        name = "Presidential Inauguration Day"
        self.assertHolidayName(
            name,
            "1999-05-29",
            "2003-05-29",
            "2007-05-29",
            "2011-05-29",
            "2015-05-29",
            "2019-05-29",
            "2023-05-29",
        )
        self.assertNoHolidayName(name, range(self.start_year, 1999))

    def test_national_day(self):
        name = "National Day"
        self.assertHolidayName(name, (f"{year}-10-01" for year in self.full_range))
        obs_dts = (
            "2017-10-02",
            "2022-10-03",
            "2023-10-02",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_christmas_day(self):
        name = "Christmas Day"
        self.assertHolidayName(name, (f"{year}-12-25" for year in self.full_range))
        obs_dts = (
            "2016-12-27",
            "2021-12-27",
            "2022-12-27",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_boxing_day(self):
        name = "Boxing Day"
        self.assertHolidayName(name, (f"{year}-12-26" for year in self.full_range))
        obs_dts = (
            "2020-12-28",
            "2021-12-28",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_prophets_birthday(self):
        name = "Id el Maulud"
        self.assertHolidayName(
            name,
            "2020-10-29",
            "2021-10-19",
            "2022-10-08",
            "2023-09-27",
            "2024-09-16",
            "2025-09-05",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)
        obs_dts = (
            "2019-11-11",
            "2022-10-10",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_eid_al_fitr(self):
        name = "Id el Fitr"
        name_holiday = f"{name} Holiday"
        self.assertHolidayName(
            name,
            "2020-05-24",
            "2021-05-13",
            "2022-05-02",
            "2023-04-21",
            "2024-04-10",
            "2025-03-30",
            "2026-03-19",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)
        self.assertIslamicNoEstimatedHolidayName(name_holiday, self.full_range)
        obs_dts = (
            "2020-05-26",
            "2025-04-01",
        )
        obs_holiday_dts = (
            "2018-06-18",
            "2023-04-24",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertHolidayName(f"{name_holiday} (observed)", obs_holiday_dts)
        self.assertNoNonObservedHoliday(obs_dts, obs_holiday_dts)

    def test_eid_al_adha(self):
        name = "Id el Kabir"
        name_holiday = f"{name} Holiday"
        self.assertHolidayName(
            name,
            "2020-07-31",
            "2021-07-20",
            "2022-07-09",
            "2023-06-28",
            "2024-06-16",
            "2025-06-06",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)
        self.assertIslamicNoEstimatedHolidayName(name_holiday, self.full_range)
        obs_dts = (
            "2017-09-04",
            "2022-07-11",
            "2024-06-18",
        )
        obs_holiday_dts = (
            "2020-08-03",
            "2022-07-12",
            "2025-06-09",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertHolidayName(f"{name_holiday} (observed)", obs_holiday_dts)
        self.assertNoNonObservedHoliday(obs_dts, obs_holiday_dts)

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2024-01-01", "New Year's Day"),
            ("2024-03-29", "Good Friday"),
            ("2024-04-01", "Easter Monday"),
            ("2024-04-10", "Id el Fitr"),
            ("2024-04-11", "Id el Fitr Holiday"),
            ("2024-05-01", "Workers' Day"),
            ("2024-06-12", "Democracy Day"),
            ("2024-06-16", "Id el Kabir"),
            ("2024-06-17", "Id el Kabir Holiday"),
            ("2024-06-18", "Id el Kabir (observed)"),
            ("2024-09-16", "Id el Maulud"),
            ("2024-10-01", "National Day"),
            ("2024-12-25", "Christmas Day"),
            ("2024-12-26", "Boxing Day"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2024-01-01", "New Year's Day"),
            ("2024-03-29", "Good Friday"),
            ("2024-04-01", "Easter Monday"),
            ("2024-04-10", "Eid al-Fitr"),
            ("2024-04-11", "Eid al-Fitr Holiday"),
            ("2024-05-01", "Workers' Day"),
            ("2024-06-12", "Democracy Day"),
            ("2024-06-16", "Eid al-Adha"),
            ("2024-06-17", "Eid al-Adha Holiday"),
            ("2024-06-18", "Eid al-Adha (observed)"),
            ("2024-09-16", "Prophet's Birthday"),
            ("2024-10-01", "National Day"),
            ("2024-12-25", "Christmas Day"),
            ("2024-12-26", "Boxing Day"),
        )
